################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
analyze_sequences.py - compute some sequence properties
=======================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

compute some sequence properties.

Usage
-----

Example::

   python analyze_sequences.py --help

Type::

   python analyze_sequences.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import tempfile
import subprocess
import optparse

import CGAT.Experiment as E
import CGAT.Genomics as Genomics

import CGAT.FastaIterator as FastaIterator
import Bio.Alphabet.IUPAC

class SequenceProperties:

    def __init__(self):
        pass

    def Load( self, in_sequence ):
        """load sequence properties from a sequence."""
        
        ## uppercase all letters
        sequence = in_sequence.upper()
        
        self.mNCodons = len(sequence) / 3
        
        self.mNStopCodons = 0

        ## setup counting arrays
        ## counts of amino acids
        self.mCountsAA = {}

        for x in Bio.Alphabet.IUPAC.extended_protein.letters: self.mCountsAA[x] = 0
        
        ## nucleotide counts for each position (is not a sum of the counts
        ## per degenerate site, as the codon might be intelligible, e.g. GNN).
        self.mCounts = [ {'A': 0, 'C': 0, 'G': 0, 'T': 0, 'X': 0, 'N': 0 },
                         {'A': 0, 'C': 0, 'G': 0, 'T': 0, 'X': 0, 'N': 0 },                         
                         {'A': 0, 'C': 0, 'G': 0, 'T': 0, 'X': 0, 'N': 0 } ]

        ## nucleotide counts for each position per degeneracy
        self.mCountsDegeneracy = []

        self.mLength = len(sequence)
        
        for x in (0,1,2):
            xx = []
            for y in range(5):
                yy = {}            
                for z in Bio.Alphabet.IUPAC.extended_dna.letters: yy[z] = 0
                xx.append( yy ) 
            self.mCountsDegeneracy.append( xx )

        for codon in [ sequence[x:x+3] for x in range(0, len(sequence), 3)]:

            for x in (0,1,2): self.mCounts[x][codon[x]] += 1

            if Genomics.IsStopCodon( codon ):
                self.mNStopCodons += 1
                continue

            try:
                aa, deg1, deg2, deg3 = Genomics.GetDegeneracy( codon ) 
                degrees = (deg1, deg2, deg3)
                for x in range(len(degrees)):
                    self.mCountsDegeneracy[x][degrees[x]][codon[x]] += 1
                self.mCountsAA[aa] += 1
                    
            except KeyError:
                pass

        self.Update()
        
    def Update( self ):
        """update fields from counts."""

        self.mNGC = 0
        self.mNSites1D, self.mNSites2D, self.mNSites3D, self.mNSites4D = 0, 0, 0, 0
        
        for x in (0,1,2):
            self.mNGC += self.mCounts[x]['C'] + self.mCounts[x]['G']
            self.mNSites1D += sum(self.mCountsDegeneracy[x][1].values() )                                
            self.mNSites2D += sum(self.mCountsDegeneracy[x][2].values() )                    
            self.mNSites3D += sum(self.mCountsDegeneracy[x][3].values() )        
            self.mNSites4D += sum(self.mCountsDegeneracy[x][4].values() )        
            
        self.mNGC3  = self.mCounts[2]['C'] + self.mCounts[2]['G']
        self.mN4DGC3 = self.mCountsDegeneracy[2][4]['C'] + self.mCountsDegeneracy[2][4]['G']

    def __str__(self):

        if self.mLength > 0:
            pngc = float(self.mNGC) / float(self.mLength)
        else:
            pngc = 0.0

        if self.mNCodons > 0:
            pngc3 = float(self.mNGC3) / float(self.mNCodons)
        else:
            pngc3 = 0.0

        if self.mNSites4D > 0:
            pn4dgc3 = float(self.mN4DGC3) / float(self.mNSites4D)
        else:
            pn4dgc3 = 0.0

        return "\t".join( ("%i" % self.mLength,
                           "%i" % self.mNCodons,                           
                           "%i" % self.mNStopCodons,
                           "%i" % self.mNSites1D,
                           "%i" % self.mNSites2D,
                           "%i" % self.mNSites3D,
                           "%i" % self.mNSites4D,
                           "%i" % self.mNGC,                           
                           "%i" % self.mNGC3,
                           "%i" % self.mN4DGC3,
                           "%5.4f" % pngc,
                           "%5.4f" % pngc3,
                           "%5.4f" % pn4dgc3,
                           ))

    def GetHeader( self ):
        return "\t".join( ("length",
                           "ncodons",
                           "nstops",
                           "nsites1d",
                           "nsites2d",
                           "nsites3d",                           
                           "nsites4d",
                           "ngc",
                           "ngc3",
                           "n4dgc3",
                           "pgc",
                           "pgc3",
                           "p4dgc3",
                           ))
    
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: analyze_sequences.py 2865 2010-03-03 10:18:28Z andreas $")
    
    parser.add_option("-s", "--species", dest="species", type="string",
                      help="species to use." )

    parser.add_option("-p", "--prefix", dest="prefix", type="string",
                      help="prefix to use for temporary files." )

    parser.set_defaults(
        filename_map = None,
        )
    
    (options, args) = E.Start( parser, add_mysql_options = True )

    iterator = FastaIterator.FastaIterator( options.stdin )

    print "id\t" + SequenceProperties().GetHeader()
    
    while 1:
        cur_record = iterator.next()
        
        if cur_record is None: break
        
        sequence = re.sub( " ", "", cur_record.sequence)
        
        if len(sequence) % 3:
            raise "sequence %s is not multiples of 3: length=%i!" % (cur_record.title, len(sequence))

        s = SequenceProperties()
        s.Load( sequence )
        
        print cur_record.title + "\t" + str(s)

    E.Stop()



####
####
##
##
## Copyright (C) 2008 Andreas Heger All rights reserved
##
## Author: Andreas Heger <heger@ebi.ac.uk>
##
## $Id: psl2map.py 2781 2009-09-10 11:33:14Z andreas $
##
##
####
####

'''
psl2map.py - build a mappping from blat alignments
==================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This scripts reads :term:`psl` formatted alignments and builds a map of queries 
to targets. The mapping can be restricted by different measures of uniqueness.

If polyA processing is turned on, the non-overlapping terminus of each read will 
be checked if they are mostly A. If they are, the query coverage will be adjusted
appropriately and the read flagged in the section polyA.

Usage
-----

Example::

   python <script_name>.py --help

Type::

   python <script_name>.py --help

for command line help.

Documentation
-------------

Code
----

'''

import sys
import re
import string
import optparse
import time
import os
import tempfile
import shutil

import CGAT.Intervals as Intervals
import CGAT.Experiment as E
import CGAT.Histogram as Histogram
import CGAT.Blat as Blat
import CGAT.IndexedFasta as IndexedFasta

def printHistogram( values, section, options, min_value = 0, increment = 1.0 ):

    if len(values) == 0:
        if options.loglevel >= 1:
            options.stdlog.write("# no histogram data for section %s\n" % (section))
        return

    outfile = open(options.output_filename_pattern % section, "w" )
    h = Histogram.Calculate( values, no_empty_bins = True, min_value = 0, increment = 1.0 )

    outfile.write("bin\t%s\n" % section)
    for bin, val in h:
        outfile.write( "%5.2f\t%i\n" % (bin, val) )
    outfile.close()

def printMatched( query_ids, section, options ):

    outfile = open(options.output_filename_pattern % section, "w" )

    for query_id in query_ids:
        outfile.write( "%s\n" % (query_id) )
    outfile.close()


def detectPolyA( query_id, matches, options, queries_fasta = None ):
    """detect PolyA tails and adjust coverage.

    polyA detection. If there are ambiguous matches, the location
    of the polyA tail is not straight-forward as suboptimal matches
    might be declared to be a tail.
    
    1. collect all matches with polyA tail - the remainder is unchanged
    2. declare polyA tail by the shortest! unaligned segment and compute
       coverage for each match appropriately. 
    
    The method checks whether the tails are consistent (always at the same end).
    If not, an AssertionError is thrown
    """
    max_total = matches[0].mQueryLength
    best_start, best_end, best_pA, best_pT, best_tail = 0, 0, 0, 0, ""

    tail_matches = []
    new_matches = []
    for match in matches:
        missing_start = match.mQueryFrom
        missing_end = match.mQueryLength - match.mQueryTo
        if missing_start < missing_end:
            smaller = missing_start
            larger = missing_end
            start,end = match.mQueryTo,match.mQueryLength
        else:
            smaller = missing_end
            larger = missing_start
            start,end = 0, match.mQueryFrom

        # check if tail is at least polyA_min_aligned and at most polyA_max_unaligned
        # are missing from the other end.
        if not(smaller < options.polyA_max_unaligned and larger > options.polyA_min_unaligned):
            new_matches.append( match )
            continue

        tail = queries_fasta.getSequence( query_id )[start:end]

        counts = {"A": 0, "T": 0, "N": 0}
        for c in tail.upper(): counts[c] = counts.get(c, 0) + 1
        total = end-start
        pA = 100.0 * (counts["A"] + counts["N"]) / total
        pT = 100.0 * (counts["T"] + counts["N"]) / total

        if options.loglevel >= 5:
            options.stdlog.write( "# polyA detection: %s:%i-%i pA=%5.2f pT=%5.2f tail=%s\n" % (query_id, start, end, pA, pT, tail))

        if max(pA,pT) < options.polyA_min_percent:
            new_matches.append( match )
            continue

        if total < max_total:
            max_total = total
            best_start, best_end, best_pA, best_pT, best_tail = start, end, pA, pT, tail

        if not( best_start == start or best_end == end ):
            if options.loglevel >= 1:
                options.stdlog.write("# inconsistent polyA tails for %s: %i-%i, %i-%i - analysis skipped\n" %\
                                         (query_id, best_start, best_end, start,end) )
            return matches

        tail_matches.append(match)

    if tail_matches:
        for match in tail_matches:
            match.mQueryCoverage += 100.0 * float(len(best_tail)) / match.mQueryLength
            assert match.mQueryCoverage <= 100.0, "%s: coverage %f > 100.0: incr=%f\n%s" % (query_id, match.mQueryCoverage, float(len(best_tail)) / match.mQueryLength, str(match))
            new_matches.append( match )

        options.outfile_polyA.write( "%s\t%i\t%i\t%i\t%5.2f\t%5.2f\t%s\n" % \
                                         (query_id, 
                                          len(tail_matches),
                                          best_start, best_end,
                                          best_pA, best_pT, best_tail ) )

    assert len(new_matches) == len(matches)

    return new_matches

def selectMatches( query_id, matches, options, queries_fasta = None ):
    """find the best match."""

    if options.loglevel >= 2:
        options.stdlog.write("# attempting to select best match for %s\n" % query_id )
        
        if options.loglevel >= 3:
            for match in matches:
                options.stdlog.write("# match=%s\n" % str(match) )

    new_matches = []

    if options.polyA:
        matches = detectPolyA( query_id, matches, options, queries_fasta )

    if options.matching_mode == "all":
        return matches, None

    elif options.matching_mode in ("best-coverage", "best-query-coverage", "best-sbjct-coverage",
                                   "best-pid", 
                                   "best-covpid", "best-query-covpid", "best-sbjct-covpid",
                                   "best-min-covpid", "best-query-min-covpid", "best-sbjct-min-covpid",):
        if options.matching_mode == "best-coverage":
            f = lambda match: min(match.mQueryCoverage, match.mSbjctCoverage)
        elif options.matching_mode == "best-query-coverage":
            f = lambda match: match.mQueryCoverage
        elif options.matching_mode == "best-sbjct-coverage":
            f = lambda match: match.mSbjctCoverage
        elif options.matching_mode == "best-pid":
            f = lambda match: match.mPid
        elif options.matching_mode == "best-covpid":
            f = lambda match: min(match.mQueryCoverage, match.mSbjctCoverage) * match.mPid
        elif options.matching_mode == "best-query-covpid":
            f = lambda match: match.mQueryCoverage * match.mPid
        elif options.matching_mode == "best-sbjct-covpid":
            f = lambda match: match.mSbjctCoverage * match.mPid
        elif options.matching_mode == "best-min-covpid":
            f = lambda match: min( (match.mQueryCoverage, match.mSbjctCoverage, match.mPid) )
        elif options.matching_mode == "best-query-min-covpid":
            f = lambda match: min(match.mQueryCoverage, match.mPid)
        elif options.matching_mode == "best-sbjct-min-covpid":
            f = lambda match: min(match.mSbjctCoverage, match.mPid)

        for match in matches:
            match.mMatchScore = f(match)

        # collect "significant" matches
        # this filter removes matches out of contention, i.e., matches
        # with very low score are not considered when assessing the uniqueness
        # of the highest scoring match
        matches.sort( lambda x,y: cmp(x.mMatchScore, y.mMatchScore) )
        matches.reverse()
        best_score = min( matches[0].mMatchScore * options.collection_threshold, 
                          matches[0].mMatchScore - options.collection_distance ) 

        for match in matches:
            ## stop when matchscore drops below best score
            if match.mMatchScore < best_score: break
            new_matches.append( match )

        if not options.keep_all_best:

            if len( new_matches ) > 1:
            
                if len(new_matches) == 2:
                    # accept matches against chrX and chrX_random (or vice versa)
                    if new_matches[0].mSbjctId == "%s_random" % new_matches[1].mSbjctId:
                        return new_matches[1:], None
                    elif new_matches[1].mSbjctId == "%s_random" % new_matches[0].mSbjctId:
                        return new_matches[:1], None
                    # or against chrUn or chrU.
                    else:
                        new_matches = [ x for x in new_matches if not( x.mSbjctId.endswith( "Un" ) or x.mSbjctId.endswith("chrU" )) ]
                        if len(new_matches) == 1:
                            return new_matches, None

                if options.ignore_all_random:
                    new_matches = [ x for x in new_matches if not( x.mSbjctId.endswith( "_random" ) or x.mSbjctId.endswith( "Un" ) or x.mSbjctId.endswith("chrU" ) )]
                    if len(new_matches) == 1:
                        return new_matches, None

                return [], "not unique: %s" % (" ".join( map(lambda x: str(x.mMatchScore), matches) ) )
        
    elif options.matching_mode == "unique":
        ## only return matches if they are "unique", i.e. no other match
        if len(matches) == 1:
            new_matches.append( matches[0] )
        else:
            return [], "not unique: %s" % (" ".join( map(lambda x: str(x.mMatchScore), matches) ) )

    matches = new_matches

    if options.best_per_sbjct:
        new_matches = []
        sbjcts = set()
        for match in matches:
            if match.mSbjctId in sbjcts: continue
            new_matches.append( match )
            sbjcts.add(match.mSbjctId)

        matches = new_matches

    return matches, None

if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: psl2map.py 2781 2009-09-10 11:33:14Z andreas $", usage = globals()["__doc__"] )

    parser.add_option("--input-filename-queries", dest="input_filename_queries", type="string",
                      help="fasta filename with queries - required for polyA analysis [%default]."  )

    parser.add_option( "--polyA", dest="polyA", action="store_true",
                       help="detect polyA tails [%default].")

    parser.add_option( "-p", "--output-filename-pattern", dest="output_filename_pattern", type="string" ,
                       help="OUTPUT filename with histogram information on aggregate coverages [%default].")

    parser.add_option( "--output-filename-empty", dest="output_filename_empty", type="string" ,
                       help="OUTPUT filename with queries for which all matches have been discarded [%default].")

    parser.add_option( "-o", "--output-format", dest="output_format", type="choice" ,
                       choices=("map", "psl" ),
                       help="output format to choose [%default].")

    parser.add_option( "-z", "--from-zipped", dest="from_zipped", action="store_true",
                       help="input is zipped.")

    parser.add_option( "--threshold-min-pid", dest="threshold_min_pid", type="float",
                       help="minimum thresholds for pid [%default]." )

    parser.add_option( "--threshold-min-matches", dest="threshold_min_matches", type="int",
                       help="minimum threshold for number of matching residues [%default]." )

    parser.add_option( "--threshold-max-error-rate", dest="threshold_max_error_rate", type="float",
                       help="maximum threshold for error of aligned part [%default]." )

    parser.add_option( "--threshold-good-query-coverage", dest="threshold_good_query_coverage", type="float",
                       help="minimum query coverage for segments to be counted as good [%default]." )

    parser.add_option( "--threshold-min-query-coverage", dest="threshold_min_query_coverage", type="float",
                       help="minimum query coverage for segments to be accepted [%default]." )

    parser.add_option( "--threshold-max-query-gapchars", dest="threshold_max_query_gapchars", type="int",
                       help="maximum number of gap characters  in query[%default]." )

    parser.add_option( "--threshold-max-query-gaps", dest="threshold_max_query_gaps", type="int",
                       help="maximum number of gaps  in query[%default]." )

    parser.add_option( "--threshold-max-sbjct-gapchars", dest="threshold_max_sbjct_gapchars", type="int",
                       help="maximum number of gap characters  in sbjct[%default]." )

    parser.add_option( "--keep-unique-matches", dest="keep_unique_matches", action="store_true",
                       help="ignore filters for unique matches [%default]." )

    parser.add_option( "--keep-all-best", dest="keep_all_best", action="store_true",
                       help="when sorting matches, keep all matches within the collection threshold [%default]." )

    parser.add_option( "--best-per-sbjct", dest="best_per_sbjct", action="store_true",
                       help="keep only the best entry per sbjct (for transcript mapping) [%default]." )

    parser.add_option( "--threshold-max-sbjct-gaps", dest="threshold_max_sbjct_gaps", type="int",
                       help="maximum number of gaps  in sbjct[%default]." )

    parser.add_option( "--test", dest="test", type="int",
                       help="test - stop after # rows of parsing[%default]." )

    parser.add_option( "-m", "--matching-mode", dest="matching_mode", type="choice",
                       choices=("best-coverage", "best-query-coverage", "best-sbjct-coverage", 
                                "best-pid", "best-covpid", "best-query-covpid", "best-sbjct-covpid", 
                                "best-min-covpid", "best-query-min-covpid", "best-sbjct-min-covpid", 
                                "unique", "all" ),
                       help="determines how to selecte the best match [%default]." )

    parser.add_option( "--filename-filter-sbjct", dest="filename_filter_sbjct", type="string",
                       help="gff file for filtering sbjct matches. Matches overlapping these regions are discarded, but see --keep-forbidden [%default]." )

    parser.add_option( "--keep-forbidden", dest="keep_forbidden", action="store_true",
                       help="if set, keep only matches that overlap the regions supplied with --filename-filter-sbjct [%default]." )

    parser.add_option( "--query-forward-coordinates", dest="query_forward_coordinates", action="store_true",
                       help="use forward coordinates for query, strand will refer to sbjct [%default]." )

    parser.add_option( "--ignore-all-random", dest="ignore_all_random", action="store_true",
                       help="if there are multiple best matches, ignore all those to chrUn and _random [%default]." )

    parser.add_option( "--collection-threshold", dest="collection_threshold", type="float",
                       help="threshold for collecting matches, percent of best score [%default]." )

    parser.add_option( "--collection-distance", dest="collection_distance", type="float",
                       help="threshold for collecting matches, difference to best score [%default]." )

    parser.set_defaults( input_filename_domains = None,
                         input_filename_queries = None,
                         threshold_good_query_coverage = 90.0,
                         threshold_min_pid = 30.0,
                         threshold_min_matches = 0,
                         threshold_max_error_rate = None,
                         output_filename_pattern = "%s",
                         keep_unique_matches = False,
                         output_format = "map",
                         print_matched = ["full", "partial", "good" ],
                         from_zipped = False,
                         combine_overlaps = True,
                         min_length_domain = 30,
                         threshold_min_query_coverage = 50,
                         min_length_singletons = 30,
                         new_family_id = 10000000,
                         add_singletons = False,
                         matching_mode = "best-coverage",
                         best_per_sbjct = False,
                         threshold_max_query_gapchars = None,
                         threshold_max_query_gaps = None,
                         threshold_max_sbjct_gapchars = None,
                         threshold_max_sbjct_gaps = None,
                         filename_filter_sbjct = None,
                         keep_forbidden = False,
                         keep_all_best = False,
                         test = None,
                         query_forward_coordinates=False,
                         output_filename_empty = None,
                         collection_threshold = 1.0, 
                         collection_distance = 0,
                         polyA = False,
                         # max residues missing from non polyA end
                         polyA_max_unaligned = 3,
                         # min residues in tail
                         polyA_min_unaligned = 10,
                         # min percent residues that are A/T in tail
                         polyA_min_percent = 70.0,
                         ## ignore duplicate matches if they are on Un or _random
                         ignore_all_random = False,
                         )

    (options, args) = E.Start( parser, add_pipe_options = True )

    if len(args) == 1:
        if options.from_zipped or args[0][-3:] == ".gz":
            import gzip
            infile = gzip.open( args[0], "r" )
        else:
            infile = open(args[0], "r" )
    else:
        infile = sys.stdin

    if options.input_filename_queries:
        queries_fasta =  IndexedFasta.IndexedFasta( options.input_filename_queries )
    else:
        queries_fasta = None
            
    if options.filename_filter_sbjct:

        try:
            import bx.intervals.io
            import bx.intervals.intersection
        except ImportError:
            raise "filtering for intervals requires the bx tools."

        intervals = GTF.readGFFFromFileAsIntervals( open( options.filename_filter_sbjct, "r" ) )

        intersectors = {}

        for contig, values in intervals.items():
            intersector = bx.intervals.intersection.Intersecter()
            for start, end in values:
                intersector.add_interval( bx.intervals.Interval(start,end) )
            intersectors[contig] = intersector

        if options.loglevel >= 1:
            options.stdlog.write("# read %i intervals for %i contigs.\n" %\
                                 (sum( [ len(x) for x in intervals.values() ] ),
                                  len( intersectors ) ))
    else:
        intersectors = None

    ################################################
    ################################################
    ################################################
    ## processing of a chunk (matches of same query)
    ################################################        
    ninput, noutput, nskipped = 0, 0, 0

    ## number of sequences with full/partial/good matches
    nfull_matches, npartial_matches, ngood_matches = 0, 0, 0
    ## number of sequences which are fully/good/partially matched
    ## i.e., after combining all aligned regions
    nfully_matched, npartially_matched, nwell_matched = 0, 0, 0
        
    nremoved_pid, nremoved_query_coverage, nempty = 0, 0, 0
    nremoved_gaps, nremoved_nmatches = 0, 0
    nremoved_regions = 0
    nqueries_removed_region = 0
    
    aggregate_coverages = []
    mapped_coverages = []
    fully_matched = []
    well_matched = []
    partially_matched = []
    new_family_id = options.new_family_id

    if options.output_filename_empty:
        outfile_empty = open( options.output_filename_empty, "w" )
        outfile_empty.write( "read_id\tcomment\n" )
    else:
        outfile_empty = None

    if options.polyA:
        options.outfile_polyA = open( options.output_filename_pattern % "polyA", "w")
        options.outfile_polyA.write( "query_id\tstart\tend\tpA+N\tpT+N\ttail\n" )

    def processChunk( query_id, matches ):
        """process a set of matches from query_id"""

        global ninput, noutput, nskipped
        global nfull_matches, npartial_matches, ngood_matches
        global nremoved_pid, nremoved_query_coverage, nempty, nremoved_gaps, nremoved_nmatches
        global nremoved_regions, nqueries_removed_region
        global outfile_empty
        ninput += 1

        full_matches = []
        good_matches = []
        partial_matches = []

        x_nremoved_pid, x_nquery_coverage, x_nremoved_gaps, x_nremoved_nmatches = 0, 0, 0, 0
        nmatches = len(matches)

        new_matches = []

        # absolute filters applicable to non-fragmentory matches

        for match in matches:

            if match.mPid < options.threshold_min_pid:
                nremoved_pid += 1
                continue
                
            if match.mNMatches < options.threshold_min_matches:
                nremoved_nmatches += 1
                continue

            if options.threshold_max_error_rate:
                r = 100.0 * math.power( options.threshold_max_error_rate, match.mNMatches + match.mNMismatches)
                if match.mPid < r:
                    nremoved_pid += 1
                    x_nremoved_pid += 1
                    continue
            
            new_matches.append(match)

        matches = new_matches

        # filter matches        
        if len(matches) == 0:
            if outfile_empty:
                outfile_empty.write( "%s\tall matches removed after applying thresholds: before=%i, npid=%i, nqcoverage=%i, ngaps=%i, nmatches=%i\n" %\
                                     (query_id, nmatches, x_nremoved_pid, x_nquery_coverage, x_nremoved_gaps, x_nremoved_nmatches ) )
            nskipped += 1
            return
        
        if options.keep_unique_matches and len(matches) == 1:
            pass
        else:
            new_matches = []

            for match in matches:

                if match.mQueryCoverage < options.threshold_min_query_coverage:
                    nremoved_query_coverage += 1
                    x_nquery_coverage += 1
                    continue

                if options.threshold_max_query_gaps and options.threshold_max_query_gaps > match.mQueryNGapsCounts:
                    nremoved_gaps += 1
                    x_nremoved_gaps += 1
                    continue

                if options.threshold_max_query_gapchars and options.threshold_max_query_gapchars > match.mQueryNGapsBases:
                    nremoved_gaps += 1
                    x_nremoved_gaps += 1
                    continue

                if options.threshold_max_sbjct_gaps and options.threshold_max_sbjct_gaps > match.mSbjctNGapsCounts:
                    nremoved_gaps += 1
                    x_nremoved_gaps += 1
                    continue

                if options.threshold_max_sbjct_gapchars and options.threshold_max_sbjct_gapchars > match.mSbjctNGapsBases:
                    nremoved_gaps += 1
                    x_nremoved_gaps += 1
                    continue
                
                new_matches.append( match )
            matches = new_matches

        if len(matches) == 0:
            if outfile_empty:
                outfile_empty.write( "%s\tall matches removed after applying thresholds: before=%i, npid=%i, nqcoverage=%i, ngaps=%i, nmatches=%i\n" %\
                                     (query_id, nmatches, x_nremoved_pid, x_nquery_coverage, x_nremoved_gaps, x_nremoved_nmatches ) )
            nskipped += 1
            return

        ## Remove queries matching to a forbidden region. This section
        ## will remove the full query if any of its matches matches in a
        ## forbidden region.
        keep = True
        for match in matches:
            if intersectors and match.mSbjctId in intersectors:
                found = intersectors[match.mSbjctId].find( match.mSbjctFrom, match.mSbjctTo )
                if found and not options.keep_forbidden or (found and not options.keep_forbidden):
                    nremoved_regions += 1
                    keep = False
                    continue

        if not keep:
            nqueries_removed_region += 1
            if outfile_empty:
                outfile_empty.write( "%s\toverlap with forbidden region\n" % query_id )
            return 

        ## check for full length matches
        for match in matches:
            if match.mQueryCoverage >= 99.9:
                full_matches.append(match)
            if match.mQueryCoverage > options.threshold_good_query_coverage:
                good_matches.append(match)
            else:
                partial_matches.append(match)
            
        if full_matches:
            nfull_matches += 1
        elif good_matches:
            ngood_matches += 1
        elif partial_matches:
            npartial_matches += 1

        ## compute coverage of sequence with matches
        intervals = []
        for match in full_matches + good_matches + partial_matches:
            intervals.append( (match.mQueryFrom, match.mQueryTo) )
        
        rest = Intervals.complement( intervals, 0, match.mQueryLength )
        
        query_coverage = 100.0 * (match.mQueryLength - sum( map( lambda x: x[1] - x[0], rest) ) ) / match.mQueryLength

        if query_coverage >= 99.9:
            fully_matched.append( query_id )
        elif  query_coverage > options.threshold_good_query_coverage:
            well_matched.append( query_id )
        else:
            partially_matched.append( query_id )

        aggregate_coverages.append( query_coverage )

        ## select matches to output
        matches, msg = selectMatches( query_id, matches, options, queries_fasta )

        if len(matches) > 0:
            for match in matches:
                if options.query_forward_coordinates:
                    match.convertCoordinates()

                if options.output_format == "map":
                    options.stdout.write( "%s\n" %\
                                              "\t".join( map(str, (
                                match.mQueryId, match.mSbjctId, 
                                match.strand,
                                "%5.2f" % match.mQueryCoverage,
                                "%5.2f" % match.mSbjctCoverage,
                                "%5.2f" % match.mPid,
                                match.mQueryLength,
                                match.mSbjctLength,
                                match.mQueryFrom, match.mQueryTo,
                                match.mSbjctFrom, match.mSbjctTo,
                                ",".join( map(str,match.mBlockSizes) ),
                                ",".join( map(str,match.mQueryBlockStarts)),
                                ",".join( map(str,match.mSbjctBlockStarts)), 
                                ))))
                elif options.output_format == "psl":
                    options.stdout.write( str(match) + "\n" )

            noutput += 1
        else:
            if outfile_empty:
                outfile_empty.write( "%s\tno matches selected: %s\n" % (query_id, msg) )
            nempty += 1
            
    if options.output_format == "map":
        options.stdout.write( "\t".join( ("query_id", "sbjct_id", "sstrand", "qcoverage","scoverage", "pid", "qlen", "slen", "qfrom", "qto", "sfrom", "sto", "blocks", "qstarts", "sstarts" ) ) + "\n" )
    elif options.output_format == "psl":
        options.stdout.write( Blat.Match().getHeader() + "\n" )

    ################################################
    ################################################
    ################################################
    ## main loop
    ################################################        
    nfully_covered = None
    matches = []
    last_query_id = None
    is_complete = True
    ninput_lines = 0

    skip = 0

    iterator = Blat.BlatIterator( infile )

    while 1:

        try:
            match = iterator.next()
        except Blat.ParsingError:
            iterator = Blat.BlatIterator( infile )
            continue
        
        if match == None: break
        
        ninput_lines += 1

        if options.test and ninput_lines > options.test:
            break
        
        if match.mQueryId != last_query_id:
            if last_query_id:
                processChunk( last_query_id, matches )
            matches = []
            last_query_id = match.mQueryId

        matches.append(match)

    processChunk( last_query_id, matches )

    printHistogram( aggregate_coverages, "aggregate", options )
    
    printHistogram( mapped_coverages, "mapped", options )

    if "full" in options.print_matched:
        printMatched( fully_matched, "full", options )

    if "good" in options.print_matched:
        printMatched( well_matched, "good", options )

    if "partial" in options.print_matched:
        printMatched( partially_matched, "partial", options )

    if options.loglevel >= 1:
        options.stdlog.write("# alignments: ninput=%i, is_complete=%s\n" % (ninput_lines, str(is_complete)))
        options.stdlog.write("# queries: ninput=%i, noutput=%i\n" % (ninput, noutput ))
        options.stdlog.write("# individual coverage: full=%i, good=%i, partial=%i\n" % (nfull_matches, ngood_matches, npartial_matches ) )
        options.stdlog.write("# aggregate  coverage: full=%i, good=%i, partial=%i\n" % (len(fully_matched), len(well_matched), len(partially_matched) ) )
        options.stdlog.write("# omitted queries: total=%i, thresholds=%i, regions=%i, selection=%i\n" %\
                             (nskipped+nqueries_removed_region+nempty,
                              nskipped, nqueries_removed_region, nempty))
        options.stdlog.write("# omitted matches: pid=%i, query_coverage=%i, gaps=%i, regions=%i, nmatches=%i\n" % (nremoved_pid, nremoved_query_coverage, nremoved_gaps, nremoved_regions, nremoved_nmatches ))

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
mali2bootstrap.py - create bootstrap samples from a mali
========================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Read a multiple alignment in write out new alignments
of the same lengths with columns sampled from the source
multiple alignment.

The default sampling is with-replacement.

Usage
-----

Example::

   python mali2bootstrap.py --help

Type::

   python mali2bootstrap.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import random
import optparse

import CGAT.Experiment as E
import CGAT.Mali as Mali

def getBootstrappedMali( mali, block_size = 1):
    """return a multiple alignment of same
    length with randomly sampled columns from mali

    block_size of 3 samples codons.
    """

    # build list of columns to extract
    l = mali.getWidth() / block_size

    columns = []
    for x in range(l):
        columns.append( random.randint(0,l-1) )
        
    return getMali( mali, columns, block_size)
    
def getSampledMali( mali, sample_size, block_size = 1 ):
    """return a multiple alignment with columns sampled
    from mali without replacement.

    block_size of 3 samples codons.
    """

    # build list of columns to sample from
    l = mali.getWidth() / block_size

    if sample_size >= l:
        raise ValueError( "sample size (%i) larger than number of columns (%i) to sample from" % (sample_size, l) )

    columns = list(range(0,l))
    random.shuffle( columns )

    return getMali( mali, columns[:sample_size], block_size)

def getMali( mali, columns, block_size = 1 ):
        
    new_mali = Mali.Mali()

    for id, val in mali.items():
        sequence = val.mString
        chars = []
        for c in columns:
            chars.append( sequence[c*block_size: c*block_size+block_size] )

        new_sequence = "".join( chars )
        new_mali.addSequence( id, 0, mali.countCharacters( new_sequence), new_sequence )

    return new_mali

##------------------------------------------------------------
if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: mali2bootstrap.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"])

    parser.add_option("-i", "--input-format", dest="input_format", type="choice",
                      choices=("plain", "fasta", "clustal", "stockholm", "phylip" ),
                      help="input format of multiple alignment"  )
    
    parser.add_option("-o", "--output-format", dest="output_format", type="choice",
                      choices=("plain", "fasta", "stockholm", "phylip"),
                      help="output format of multiple alignment"  )

    parser.add_option( "-p", "--output-filename-pattern", dest="output_filename_pattern", type="string",
                       help="pattern for output filenames. Should contain a %(id)i. If not given, the output is to stdout with --separator [default=%default]."  )

    parser.add_option("-n", "--samples", dest="samples", type="int",
                      help="number of samples."  )

    parser.add_option("-r", "--no-replacement", dest="no_replacement", type="int",
                      help="sample without replacement. The parameter gives the size of the multiple alignment [default=%default]."  )
    
    parser.add_option("-b", "--block-size", dest="block_size", type="int",
                      help="block size. Use 3 for sampling from codons."  )

    parser.set_defaults(
        input_format="fasta",
        output_format="fasta",
        samples = 10,
        block_size = 1,
        output_filename_pattern = None,
        no_replacement = None,
        separator = "//")

    (options, args) = E.Start( parser )

    mali = Mali.Mali()

    mali.readFromFile( sys.stdin, format = options.input_format )

    for x in range(options.samples):
        
        if options.no_replacement != None:
            new_mali = getSampledMali( mali, options.no_replacement, options.block_size )
        else:
            new_mali = getBootstrappedMali( mali, options.block_size )

        if options.output_filename_pattern:

            filename = options.output_filename_pattern % { "id": x+1}
            target_directory = os.path.dirname( filename )
            if not os.path.exists( target_directory ):
                os.makedirs( target_directory )
            outfile = open( filename, "w" )
            E.info( "creating mali %s" % filename )
        else:
            outfile = options.stdout

        new_mali.writeToFile( outfile, format = options.output_format )

        if outfile == sys.stdout:
            if options.separator and x < options.samples - 1:
                options.stdout.write( options.separator + "\n" )
        else:
            outfile.close()

    E.Stop()
        



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
gff2histogram.py - compute histograms from intervals in gff or bed format
=========================================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script computes distribution interval sizes, intersegmental distances
and interval overlap from a list of intervals in :term:`gff` or :term:`bed` 
format. 

The output will be written into separate files. Filenames are given by 
``--ouput-filename-pattern``.

Available methods are:

hist
    Output a histogram of interval sizes and distances between intervals
    in nucleotides.

stats
    Output summary statistics of interval sizes and distances between
    intervals

values
    Output distances, sizes, and overlap values to separate files.

all
    all of the above.

Usage
-----

Example::

   python gff2histogram.py < in.bed 

Type::

   python gff2histogram.py --help

for command line help.

Documentation
-------------

Code
----

'''
import sys
import string
import re
import optparse

USAGE="""python %s [OPTIONS] input1 input2

compute simple statistics on the size of features and the 
distance between features in a gff file.

Methods:
  hist: histogram of distance/sizes
  stats: descriptive statistics of distance/sizes
  overlaps: output overlapping features

Version: $Id: gff2histogram.py 2781 2009-09-10 11:33:14Z andreas $
""" % sys.argv[0]

import CGAT.Experiment as E
import CGAT.GTF as GTF
import CGAT.Bed as Bed
import CGAT.Histogram as Histogram
import CGAT.Stats as Stats

##------------------------------------------------------------------------
if __name__ == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: gff2histogram.py 2781 2009-09-10 11:33:14Z andreas $")

    parser.add_option("-b", "--bin-size", dest="bin_size", type="string",
                      help="bin size."  )
    parser.add_option("--min-value", dest="min_value", type="float",
                      help="minimum value for histogram.")
    parser.add_option("--max-value", dest="max_value", type="float",
                      help="maximum value for histogram.")
    parser.add_option("--no-empty-bins", dest="no_empty_bins", action="store_true",
                      help="do not display empty bins.")
    parser.add_option("--with-empty-bins", dest="no_empty_bins", action="store_false",
                      help="display empty bins.")
    parser.add_option("--ignore-out-of-range", dest="ignore_out_of_range", action="store_true",
                      help="ignore values that are out of range (as opposed to truncating them to range border.")
    parser.add_option("--missing", dest="missing_value", type="string",
                      help="entry for missing values [%default]." )
    parser.add_option("--dynamic-bins", dest="dynamic_bins", action="store_true",
                      help="each value constitutes its own bin." )
    parser.add_option( "--format", dest="format", type="choice", 
                       choices=( "gff", "gtf", "bed" ),
                       help="input file format [%default].")
    parser.add_option( "--method", dest="methods", type="choice", action="append",
                       choices=( "all", "hist", "stats", "overlaps", "values" ),
                       help="methods to apply [%default].")
    parser.add_option( "--data", dest="data", type="choice",
                       choices=( "all", "size", "distance" ),
                       help="data to compute [%default].")

    parser.set_defaults(
        no_empty_bins = True,
        bin_size = None,
        dynamic_bins = False,
        ignore_out_of_range = False,
        min_value = None,
        max_value = None,
        nonull = None,
        missing_value = "na",
        output_filename_pattern="%s",
        methods = [],
        data = "all",
        format = "gff",
        )

    (options, args) = E.Start( parser, add_output_options = True )

    if "all" in options.methods:
        options.methods = ("hist", "stats", "overlaps")
        if not options.output_filename_pattern: options.output_filename_pattern = "%s"

    if len(options.methods) == 0:
        raise ValueError( "please provide counting method using --method option" )

    if options.format in ( "gff", "gtf" ):
        gffs = GTF.iterator( options.stdin )
    elif options.format == "bed":
        gffs = Bed.iterator( options.stdin )

    values_between = []
    values_within = []
    values_overlaps = []

    if "overlaps" in options.methods:
        if not options.output_filename_pattern: 
            options.output_filename_pattern = "%s"
        outfile_overlaps = E.openOutputFile( "overlaps" )
    else:
        outfile_overlaps = None

    last = None
    ninput, noverlaps = 0,0
    for this in gffs:
        ninput += 1
        values_within.append( this.end - this.start )

        if last and last.contig == this.contig:
            if this.start < last.end:
                noverlaps += 1
                if outfile_overlaps:
                    outfile_overlaps.write( "%s\t%s\n" % (str(last), str(this)) )
                values_overlaps.append( min(this.end, last.end) - max(last.start, this.start) )
                if this.end > last.end:
                    last = this
                continue
            else:
                values_between.append( this.start - last.end )
                # if this.start - last.end < 10: 
                #     print str(last)
                #     print str(this)
                #     print "=="
                values_overlaps.append( 0 )

        last = this

    if "hist" in options.methods:
        outfile = E.openOutputFile( "hist" )
        h_within = Histogram.Calculate( values_within,
                                        no_empty_bins = options.no_empty_bins,
                                        increment = options.bin_size,
                                        min_value = options.min_value,
                                        max_value = options.max_value,
                                        dynamic_bins = options.dynamic_bins,
                                        ignore_out_of_range = options.ignore_out_of_range )

        h_between = Histogram.Calculate( values_between,
                                         no_empty_bins = options.no_empty_bins,
                                         increment = options.bin_size,
                                         min_value = options.min_value,
                                         max_value = options.max_value,
                                         dynamic_bins = options.dynamic_bins,
                                         ignore_out_of_range = options.ignore_out_of_range )

        if "all" == options.data:
            outfile.write( "residues\tsize\tdistance\n" )
            combined_histogram = Histogram.Combine( [h_within, h_between], missing_value = options.missing_value )
            Histogram.Write( outfile, combined_histogram, nonull = options.nonull )        
        elif options.data == "size":
            outfile.write( "residues\tsize\n" )
            Histogram.Write( outfile, h_within, nonull = options.nonull )        
        elif options.data == "distance":
            outfile.write( "residues\tdistance\n" )
            Histogram.Write( outfile, h_between, nonull = options.nonull )        

        outfile.close()

    if "stats" in options.methods:
        outfile = E.openOutputFile( "stats" )
        outfile.write( "data\t%s\n" % Stats.Summary().getHeader() )
        if options.data in ("size", "all"):
            outfile.write( "size\t%s\n" % str(Stats.Summary(values_within)) )
        if options.data in ("distance", "all"):
            outfile.write( "distance\t%s\n" % str(Stats.Summary(values_between)) )
        outfile.close()

    if "values" in options.methods:
        outfile = E.openOutputFile( "distances" )
        outfile.write( "distance\n%s\n" % "\n".join( map(str, values_between) ) )
        outfile.close()
        outfile = E.openOutputFile( "sizes" )
        outfile.write( "size\n%s\n" % "\n".join( map(str, values_within) ) )
        outfile.close()
        outfile = E.openOutputFile( "overlaps" )
        outfile.write( "overlap\n%s\n" % "\n".join( map(str, values_overlaps) ) )
        outfile.close()

    E.info( "ninput=%i, ndistance=%i, nsize=%i, noverlap=%i" % (ninput, 
                                                                len(values_between),
                                                                len(values_within),
                                                                noverlaps) )

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
r_compare_distributions.py - statistical test for distributions
===============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script compares two distributions from two samples
of data. If only one sample is given, the sample can be
compared the normal distribution.

The tests are implemented in ``R``.

Usage
-----

Example::

   python r_compare_distributions.py --help

Type::

   python r_compare_distributions.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import time
import random
import rpy2
from rpy2.robjects import r as R

"""Tests to compare distribution of values

Input: two sets of values. These can be either given
as values directly or as categories, which will be mapped to
values.

Question: is the distribution of values different?

Implementation: uses R 

See http://www.graphpad.com/library/BiostatsSpecial/article_197.htm
for an interesting article about normality tests.


"""

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import CGAT.Stats as Stats

if __name__  == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: r_compare_distributions.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option( "-m", "--method", dest="method", type="choice",
                       help="method to use: ks=Kolmogorov-Smirnov, mwu=Mann-WhitneyU, shapiro=Shapiro-Wilk, paired-mwu=paired Mann-WhitneyU, paired-t=paired t-test [default=%default]",
                       choices=("ks", "mwu", "shapiro", "paired-mwu", "paired-t" ) )
    parser.add_option( "-a", "--hardcopy", dest="hardcopy", type="string",
                      help="write hardcopy to file.", metavar = "FILE" )
    parser.add_option( "-1", "--infile1", dest="filename_input1", type="string" ,
                       help="input filename for distribution 1.")
    parser.add_option( "-2", "--infile2", dest="filename_input2", type="string" ,
                       help="input filename for distribution 2.")
    parser.add_option( "--legend", dest="legend", type="string" ,
                       help="legend for histograms.""")
    parser.add_option( "-f", "--infile-map", dest="filename_input_map", type="string" ,
                       help="input filename for mapping categories to values.")
    parser.add_option( "-n", "--norm-test", dest="norm_test", action="store_true",
                       help="""test if a set of values is normally distributed. Mean and variance
                       are calculated from the data.""")
    parser.add_option( "-b", "--num-bins", dest="num_bins", type="int",
                       help="""number of bins (for plotting purposes only).""")
    parser.add_option( "--bin-size", dest="bin_size", type="float",
                       help="""bin size for plot.""")
    parser.add_option( "--min-value", dest="min_value", type="float",
                       help="""minimum_value for plot.""")
    parser.add_option( "--max-value", dest="max_value", type="float",
                       help="""maximum_value for plot.""")
    parser.add_option( "--skip-plot", dest="plot", action="store_false",
                       help="""skipping plotting.""")
    parser.add_option( "--header", dest="header", type="string",
                       help="""header of value column [default=%default].""")
    parser.add_option( "--title", dest="title", type="string",
                       help="""plot title [default=%default].""")


    parser.set_defaults(
        method = "ks",
        filename_input1 = None,
        filename_input2 = None,
        filename_input_map = None,
        legend = None,
        norm_test = False,
        num_bins = 0,
        legend_range = "2,2",
        bin_size = None,
        min_value = None,
        plot = True,
        header = "value",
        title = None,
        )
    
    (options, args) = E.Start( parser,
                               add_pipe_options = True,
                               add_psql_options = True,)

    kwargs = {}
    xargs = []
    for arg in args:
        if "=" in arg:
            key,value = arg.split("=")
            kwargs[key] = value
        else:
            xargs.append( arg )

    if options.legend: options.legend = options.legend.split(",")
    
    map_category2value = {}
    if options.filename_input_map:
        map_category2value = IOTools.ReadMap( open(options.filename_input_map, "r"),
                                              map_functions=(str,float))
        f = str
    else:
        f = float

    if options.filename_input1:
        infile1 = open(options.filename_input1, "r")
    else:
        infile1 = sys.stdin

    values1, errors1 = IOTools.ReadList( infile1,
                                         map_function=f,
                                         map_category=map_category2value )

    if options.filename_input1:
        infile1.close()

    if errors1 and options.loglevel >= 3:
        options.stdlog.write("# errors in input1: %s\n" % ";".join(map(str, errors1)))

    if options.norm_test:
        mean = R.mean(values1)
        stddev = R.sd(values1)
        options.stdlog.write( "# creating %i samples from normal distribution with mean %f and stddev %f\n" % (len(values1), mean, stddev))
        
        values2 = R.rnorm( len(values1), mean, stddev )
        errors2 = ()
    else:
        values2, errors2 = IOTools.ReadList( open(options.filename_input2, "r"),
                                             map_function=f,                                             
                                             map_category=map_category2value )    

    if errors2 and options.loglevel >= 3:
        options.stdlog.write("# errors in input2: %s\n" % ";".join(map(str, errors2)))

    if options.loglevel >= 1:
        options.stdlog.write( "# ninput1=%i, nerrors1=%i, ninput2=%i, nerrors2=%i\n" % (len(values1), len(errors1),
                                                                                        len(values2), len(errors2)) )

    if options.method in ("paired-mwu", "paired-t"):
        if len(values1) != len(values2):
            raise ValueError("number of values must be equal for paired tests.")

    if options.hardcopy:
        R.png(options.hardcopy, width=1024, height=768)

    if options.method == "ks":
        result = R.ks_test( values1, values2, *xargs, **kwargs )
    elif options.method == "mwu":
        result = R.wilcox_test( values1, values2, paired=False, correct = True, *xargs, **kwargs)
    elif options.method == "paired-mwu":
        result = R.wilcox_test( values1, values2, paired=True, correct = True, *xargs, **kwargs)
    elif options.method == "paired-t":
        result = R.t_test( values1, values2, paired=True, *xargs, **kwargs )
    elif options.method == "shapiro":
        if len(values1) > 5000:
            E.warn("shapiro-wilk test only accepts < 5000 values, a random sample has been created." )
            values1 = random.sample( values1, 5000 )
        result = R.shapiro_test( values1, *xargs, **kwargs )

    if options.plot:
        R.assign("v1", values1)
        R.assign("v2", values2)    

        if options.title:
            # set the size of the outer margins - the title needs to be added at the end
            # after plots have been created
            R.par(oma=R.c(0,0,4,0) )                     

        R.layout(R.matrix((1,2,3,4), 2, 2, byrow = True))

        R.boxplot( values1, values2, col=('white','red'), main="Boxplot" )
        R("""qqplot( v1, v2, main ='Quantile-quantile plot' ); lines( c(0,1), c(0,1) );""")

        ## compute breaks:

        min_value = min( min(values1), min(values2) )
        if options.min_value != None:
            min_value = min( min_value, options.min_value)

        max_value = max( max(values1), max(values2) )
        if options.max_value != None:
            max_value = max( max_value, options.max_value)

        extra_options = ""
        if options.num_bins and not (options.min_value or options.max_value):
            extra_options += ", breaks=%i" % options.num_bins

        elif options.num_bins and (options.min_value or options.max_value):
            bin_size = float((max_value - min_value )) / (options.num_bins + 1)
            breaks = [ min_value + x * bin_size for x in range(options.num_bins) ]
            extra_options += ", breaks=c(%s)" % ",".join(map( str, breaks) )

        elif options.bin_size != None:
            num_bins = int(((max_value - min_value) / options.bin_size)) + 1
            breaks = [ min_value + x * options.bin_size for x in range(num_bins+1) ]
            extra_options += ", breaks=c(%s)" % ",".join(map( str, breaks) )

        R("""h1 <- hist( v1, freq=FALSE,           density=20, main='Relative frequency histogram' %s)""" % extra_options )
        R("""h2 <- hist( v2, freq=FALSE, add=TRUE, density=20, col='red', offset=0.5, angle=135 %s)""" % extra_options)
        if options.legend:
            R("""legend( ( max(c(h1$breaks[-1], h2$breaks[-1])) - min(c(h1$breaks[1], h2$breaks[1]) ) ) / 2,
            max( max(h1$density), max(h2$density)) / 2, c('%s'), fill=c('white','red'))""" % (
                "','".join(options.legend)) )

        R("""h1 <- hist( v1, freq=TRUE,            density=20, main='Absolute frequency histogram' %s)""" % extra_options )
        R("""h2 <- hist( v2, freq=TRUE,  add=TRUE, density=20, col='red', offset=0.5, angle=135 %s )""" % extra_options)
        if options.legend:    
            R("""legend( ( max(c(h1$breaks[-1], h2$breaks[-1])) - min(c(h1$breaks[1], h2$breaks[1]) ) ) / 2,
            max( max(h1$counts), max(h2$counts)) / 2, c('%s'), fill=c('white','red'))""" % (
                "','".join(options.legend)) )

        if options.title:
            R.mtext( options.title,3,outer=True,line=1,cex=1.5)  

    if options.loglevel >= 1:
        options.stdout.write("## Results for %s\n" % result['method'])

    options.stdout.write( "%s\t%s\n" % ("key", options.header) )

    for key in result.keys():
        if key == "data.name": continue
        options.stdout.write("\t".join( (key, str(result[key]))) + "\n")

    stat = Stats.Summary(values1)
    for key, value in stat.items():
        options.stdout.write( "%s1\t%s\n" % (str(key), str(value)))

    stat = Stats.Summary(values2)
    for key, value in stat.items():
        options.stdout.write( "%s2\t%s\n" % (str(key), str(value)))

    if options.plot:
        if options.hardcopy:
            R.dev_off()

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
analyze_readpositions.py - compute read coverage on transcripts
===============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------


input = stream of counts on a transcript

Usage
-----

Example::

   python analyze_readpositions.py --help

Type::

   python analyze_readpositions.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile

USAGE=""" program $Id: analyze_readpositions.py 2781 2009-09-10 11:33:14Z andreas $


"""
import CGAT.Experiment as E
import CGAT.CSV as CSV
import CGAT.IOTools as IOTools
import CGAT.Stats as Stats
import numpy

def main():

    parser = E.OptionParser( version = "%prog version: $Id: analyze_readpositions.py 2781 2009-09-10 11:33:14Z andreas $", usage = globals()["__doc__"])

    parser.add_option( "--output-filename-pattern", dest="output_filename_pattern", type="string",
                      help="pattern for additional output files [%default]."  )

    parser.set_defaults(
        length=1000,
        minimum_coverage=0.90,
        maximum_reads = [1,10,20,50,100],
        output_filename_pattern = "%s",
        normalize = True,
        )

    (options, args) = E.Start( parser, add_csv_options = True )

    fields, table = CSV.ReadTable( sys.stdin, dictreader=CSV.DictReaderLarge )
    
    map_fields2column = {}
    for x in fields: map_fields2column[x] = len(map_fields2column)
    
    coverage_5prime = numpy.zeros( options.length, numpy.float )
    coverage_3prime = numpy.zeros( options.length, numpy.float )

    coverage_maxreads5prime = numpy.zeros( options.length, numpy.float )
    coverage_maxreads3prime = numpy.zeros( options.length, numpy.float )

    coverage_full5prime = numpy.zeros( options.length, numpy.float )
    coverage_full3prime = numpy.zeros( options.length, numpy.float )

    coverage_min5prime = numpy.zeros( options.length, numpy.float )
    coverage_min3prime = numpy.zeros( options.length, numpy.float )
    
    histograms = []
    for x in range(len( options.maximum_reads) ):
        histograms.append( 
            [ numpy.zeros( options.length, numpy.float ),
              numpy.zeros( options.length, numpy.float ),
              0 ] )

    ninput, noutput, nfull, nmincov, nskipped, nlength, nmaxreads = 0, 0, 0, 0, 0, 0, 0
    for row in table:
        length, covered, meancov, data, nreads = (int(row["cov_nval"]), 
                                                          float(row["cov_covered"]), 
                                                          float(row["cov_mean"]), 
                                                          row["cov_values"],
                                                          int(row["nover2"]) )
        ninput += 1
        if length < options.length: 
            nlength += 1
            continue
        
        if data == "na":
            nskipped += 1
            continue

        noutput += 1
        mincov = covered / length
        values = map( float, data.split(";") )
        m = max(values)
        values = [ x / m for x in values ]
        coverage_5prime += values[0:1000]
        coverage_3prime += values[-1000:]
        
        if mincov >= 1.0:
            coverage_full5prime += values[0:1000]
            coverage_full3prime += values[-1000:]
            nfull += 1

        if meancov >= options.minimum_coverage:
            coverage_min5prime += values[0:1000]
            coverage_min3prime += values[-1000:]
            nmincov += 1

        for maxreads in range( len(options.maximum_reads) ):
            if nreads <= options.maximum_reads[maxreads]:
                histograms[maxreads][0] += values[0:1000]
                histograms[maxreads][1] += values[-1000:]
                histograms[maxreads][2] += 1

    if options.normalize:
        for x5,x3 in ((coverage_5prime,
                       coverage_3prime),
                      (coverage_min5prime,
                       coverage_min3prime),
                      (coverage_full5prime,
                       coverage_full3prime) ):
            m = max( (max(x5), max(x3)) )
            x3 /= m
            x5 /= m

        for x5,x3,c in histograms:
            m = max( (max(x5), max(x3) ) )
            x5 /= m
            x3 /= m
            
    outfile = options.stdout
    outfile.write( "\t".join( ("distance", "minlen-5'", "minlen-3'", "mincov-5'", "mincov-3'", "full-5'", "full-3'" ) ) + "\n" )

    for x in range(0, options.length):
        outfile.write( "\t".join( [ "%6.4f" % x for x in \
                                        (x, 
                                         coverage_5prime[x],
                                         coverage_3prime[x],
                                         coverage_min5prime[x],
                                         coverage_min3prime[x],
                                         coverage_full5prime[x],
                                         coverage_full3prime[x] ) ] ) + "\n" )


    outfile5 = open(options.output_filename_pattern % "reads5", "w")
    outfile3 = open(options.output_filename_pattern % "reads3", "w")

    outfile5.write( "\t".join( ["distance",] + ["reads%i" % options.maximum_reads[y] for y in range(len(options.maximum_reads) ) ] ) + "\n" )
    outfile3.write( "\t".join( ["distance",] + ["reads%i" % options.maximum_reads[y] for y in range(len(options.maximum_reads) ) ] ) + "\n" )
    for x in range(0, options.length):
        outfile5.write( "%i\t%s\n" % (x, "\t".join( [ "%6.4f" % histograms[y][0][x] for y in range(len(options.maximum_reads) ) ] )))
        outfile3.write( "%i\t%s\n" % (x, "\t".join( [ "%6.4f" % histograms[y][1][x] for y in range(len(options.maximum_reads) ) ] )))

    E.info( "ninput=%i, noutput=%i, nmaxreads=%i, nfull=%i, nmincov=%i, nskipped=%i, nlength=%i" %\
                (ninput, noutput, nmaxreads, nfull, nmincov, nskipped, nlength) )
    
    E.Stop()

if __name__ == "__main__":
    sys.exit( main() )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
compare_clusters.py - compare two partitions of the same data
=============================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Input: two maps.

Method: build a graph between clusters in the two partitions.
Clusters are connected, if they share common identifiers.

Usage
-----

Example::

   python compare_clusters.py --help

Type::

   python compare_clusters.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import time
import random
import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import networkx

def getFile( section, options ):
    if options.output_pattern:
        if options.loglevel >= 1:
            options.stdlog.write( "# output for section %s goes to %s\n" % (section, options.output_pattern % section))
        return open(options.output_pattern % section, "w" )
    else:
        return options.stdout

if __name__  == "__main__":

    parser = E.OptionParser( version = "%prog version: $Id: compare_clusters.py 2782 2009-09-10 11:40:29Z andreas $")

    parser.add_option( "-o", "--output-pattern", dest="output_pattern", type="string" ,
                       help="output pattern for filenames.")

    parser.set_defaults(
        output_pattern = None,
        format = "%5.2f",
        )
    
    (options, args) = E.Start( parser,
                                        add_pipe_options = True )

    if len(args) != 2:
        raise "please supply to filenames with the clusters."

    map_id2cluster1, map_cluster2ids1 = IOTools.ReadMap( open( args[0] ), both_directions = True )
    map_id2cluster2, map_cluster2ids2 = IOTools.ReadMap( open( args[1] ), both_directions = True )

    graph = networkx.Graph()
    
    for a in map_cluster2ids1.keys(): graph.add_node( (1,a) )
    for b in map_cluster2ids2.keys(): graph.add_node( (2,b) )

    ## build graph between clusters
    for cluster1, ids1 in map_cluster2ids1.items():
        for id1 in ids1:
            if id1 in map_id2cluster2:
                graph.add_edge( (1,cluster1), (2,map_id2cluster2[id1]) )

    components = networkx.connected_components( graph )

    #######################################################
    #######################################################
    #######################################################    
    ## write components and compute counts
    #######################################################    
    outfile = getFile( "components", options )
    outfile.write("id\ttotal\tn1\tn2\tmembers1\tmembers2\n" )
    n = 0
    counts = {}
    subsets = []
    for component in components:

        m1, m2 = [], []
        
        for x in component:
            if x[0] == 1:
                m1.append(x[1])
            else:
                m2.append(x[1])
                
        t = len(component)
        n1 = len(m1)
        n2 = len(m2)
        cc = ( n1, n2 )
        if cc not in counts: counts[cc] = 0
        counts[cc] += 1
        
        if cc == (1,1): subsets.append( n )
        
        n += 1
        outfile.write( "%i\t%i\t%i\t%i\t%s\t%s\n" % (n, t, n1, n2, ",".join(m1), ",".join(m2) ))

    if outfile != options.stdout:
        outfile.close()
    else:
        outfile.write ("//\n")

    #######################################################
    #######################################################
    #######################################################    
    ## write counts
    #######################################################    
    outfile = getFile( "counts", options )
    outfile.write("n1\tn2\tcounts\tpcounts1\tpcounts2\n" )
    for cc, c in counts.items():
        outfile.write("%i\t%i\t%i\t%s\t%s\n" % ( cc[0], cc[1], c,
                                                 options.format % (100.0 * float(c) / len(map_cluster2ids1)),
                                                 options.format % (100.0 * float(c) / len(map_cluster2ids2)) ))

    if outfile != options.stdout:
        outfile.close()
    else:
        outfile.write ("//\n")

    #######################################################
    #######################################################
    #######################################################    
    ## analyze subsets - how many of the 1:1 clusters
    ## contain the exact members?
    #######################################################    
    outfile = getFile( "subsets", options )
    outfile.write("id\tn1\tn2\tunion\tinter\tunique1\tunique2\n" )    

    ntrue = 0
    nrest1 = 0
    nrest2 = 0
    nother = 0
    
    for component_id in subsets:
        component = components[component_id]
        if component[0][0] == 1:
            id1, id2 = component[0][1], component[1][1]
        else:
            id1, id2 = component[1][1], component[0][1]

        members1 = set(map_cluster2ids1[id1])
        members2 = set(map_cluster2ids2[id2])

        union = len(members1.union(members2))
        intersection = len(members1.intersection(members2))
        rest1 = len(members1.difference(members2))
        rest2 = len(members2.difference(members1))

        if rest1 == 0 and rest2 == 0:
            ntrue += 1
        elif rest1 == 0:
            nrest1 += 1
        elif rest2 == 0:
            nrest2 += 1
        else:
            nother += 1
        
        outfile.write( "%i\t%i\t%i\t%i\t%i\t%i\t%i\n" % ( component_id,
                                                          len(members1),
                                                          len(members2),
                                                          union,
                                                          intersection,
                                                          rest1, rest2))

    if outfile != options.stdout:
        outfile.close()
    else:
        outfile.write ("//\n")

    ## write subset statistics
    ntotal = len(subsets)
    options.stdout.write("# subset statistics of 1:1 corresponding clusters\n" )
    options.stdout.write( "class\tcounts\ttotal\n")
    options.stdout.write( "%s\t%i\t%s\n" % ("total", ntotal, options.format % 100 ))
    options.stdout.write( "%s\t%i\t%s\n" % ("true", ntrue, options.format % (100.0 * ntrue / ntotal )))
    options.stdout.write( "%s\t%i\t%s\n" % ("unique1", nrest1, options.format % (100.0 * nrest1 / ntotal )))
    options.stdout.write( "%s\t%i\t%s\n" % ("unique2", nrest2, options.format % (100.0 * nrest2 / ntotal )))
    options.stdout.write( "%s\t%i\t%s\n" % ("other", nother, options.format % (100.0 * nother / ntotal )))            

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
convert adjacency lists to edge lists
=====================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

convert a graph in table format into a graph 
with links.

A table in graph format stores adjacency lists for each node, for example::

   node1   node2;node3;node4   value1;value2;value3

This will be converted to::

   node1   node2   value1
   node1   node3   value2
   node1   node4   value3

Usage
-----

Type::

   python <script_name>.py --help

for command line help.

Code
----

""" 

import os
import sys
import re
import optparse

import CGAT.Experiment as E

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id$", usage = globals()["__doc__"] )
    
    parser.add_option("-e", "--headers", dest="headers", type="string",
                      help="',' separated list of node headers [default=%default]."  )

    parser.set_defaults(
        headers="node1,node2",
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    ## do sth
    ninput, nskipped, noutput = 0, 0, 0

    first = True
    for line in options.stdin:
        if line.startswith( "#" ): continue
        data = line[:-1].split("\t")
        if first:
            headers = options.headers.split(",")
            if len(data) >= 2: extra = "\t%s" % ("\t".join(data[2:]))
            else: extra = ""
            options.stdout.write("%s%s\n" % ("\t".join(headers), extra))
            first = False
            continue

        ninput += 1
        if len(data) < 2: continue
        values = [ x.split(";") for x in data[1:] if x != ""]
        if len(values) == 0: 
            nskipped += 1
            continue

        l = min([len(x) for x in values])
        assert l == max([len(x) for x in values]), "unequal number of fields in line '%s'" % line[:-1]
        node1 = [[data[0]]*l]
        for n in zip( *(node1 + values) ):
            options.stdout.write("\t".join(n) + "\n" )
            noutput += 1

    E.info( "ninput=%i, noutput=%i, nskipped=%i" % (ninput, noutput,nskipped) )

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )



 ################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: bed2graph.py 2861 2010-02-23 17:36:32Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
"""
bed2graph.py - compute the overlap graph between two bed files
==============================================================

:Author: Andreas Heger
:Release: $Id: bed2graph.py 2861 2010-02-23 17:36:32Z andreas $
:Date: |today|
:Tags: Python

Purpose
-------

This script ouputs a list of the names of all overlapping intervals 
between two bed files.

Usage
-----

Type::

   python bed2graph.py A.bed.gz B.bed.gz > graph.out

for command line help.

Code
----

""" 

import os
import sys
import re
import optparse

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import CGAT.Bed as Bed
import numpy

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: bed2graph.py 2861 2010-02-23 17:36:32Z andreas $", usage = globals()["__doc__"] )

    parser.add_option("-o", "--output", dest="output", type="choice",
                      choices = ("full", "name"),
                      help="output either ``full`` overlapping entries, only the ``name``s. [default=%default]." )

    parser.set_defaults(
        output = "full",
        )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    if len(args) != 2:
        raise ValueError( "two arguments required" )

    if args[0] == "-":
        infile1 = options.stdin
    else:
        infile1 = IOTools.openFile( args[0], "r")

    infile2 = IOTools.openFile(args[1], "r")        
    
    idx = Bed.readAndIndex( infile2, with_values = True )

    output = options.output
    outfile = options.stdout
    
    if output == "name":
        outfile.write( "name1\tname2\n" )
        outf = lambda x: x.fields[0]
    else:
        outf = str
        
    for bed in Bed.iterator( infile1 ):
        try:
            overlaps = idx[bed.contig].find( bed.start, bed.end )
        except (KeyError, IndexError):
            # ignore missing contig and zero length intervals
            continue

        for o in overlaps:
            outfile.write( "\t".join( (outf(bed), outf(o[2]))) + "\n" )

    E.Stop()


if __name__ == "__main__":
    sys.exit( main( sys.argv) )



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
codemls2tsv.py - analyze results from site-specific codeml run
=======================================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

The input is either:

   * filenames with a set of files of related codeml runs

summary-numbers:

jalview: annotation file for Jalview

Usage
-----

Example::

   python codemls2tsv.py --help

Type::

   python codemls2tsv.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import optparse
import time

import CGAT.Genomics as Genomics
import CGAT.Experiment as E
import CGAT.WrapperCodeML as WrapperCodeML
import CGAT.Stats as Stats
import CGAT.Mali as Mali
import alignlib

def selectPositiveSites( results, selection_mode, options, mali = None ):
    """returns sites, which are consistently estimated to be positively selected.

    Depending on the option selection_mode, various sites are selected:

    'all': all positive sites are returned
    'consistent': only positive sites that are positive in all models and runs
    'emes': only sites that are > 0.9 in one model and at least > 0.5 in all other models

    If mali is given, positions that are not fully aligned are removed.

    """

    ## filter and extract functions
    if selection_mode == "emes":
        filter_f = lambda x: x.mProbability >= 0.5 and x.mOmega >= options.filter_omega
    else:
        filter_f = lambda x: x.mProbability >= options.filter_probability and x.mOmega >= options.filter_omega
        
    extract_f = lambda x: x.mResidue

    ## maximum significance per site (for emes)
    max_per_site = {}

    total_sites = set()

    first = True
    
    for result in results:

        for model in options.models:
        
            sites = result.mSites[model]
            
            s1, s2 = set(), set()
            if "neb" in options.analysis:
                s1 = set(map( extract_f, filter( filter_f, sites.mNEB.mPositiveSites)))
                for x in filter( filter_f, sites.mNEB.mPositiveSites):
                    if x.mResidue not in max_per_site: max_per_site[x.mResidue] = 0
                    max_per_site[x.mResidue] = max( x.mProbability, max_per_site[x.mResidue] )
                
            if "beb" in options.analysis:
                s2 = set(map( extract_f, filter( filter_f, sites.mBEB.mPositiveSites)))
                for x in filter( filter_f, sites.mBEB.mPositiveSites):
                    if x.mResidue not in max_per_site: max_per_site[x.mResidue] = 0
                    max_per_site[x.mResidue] = max( x.mProbability, max_per_site[x.mResidue] )


            s = s1.union( s2 )
            
            if first:
                total_sites = s
                first = False
            else:
                if selection_mode == "all":
                    total_sites = total_sites.union( s )
                elif selection_mode == "consistent":
                    total_sites = total_sites.intersection( s )
                elif selection_mode == "emes":
                    total_sites = total_sites.intersection( s )


    if selection_mode == "emes":
        if options.loglevel >= 2:
            options.stdlog.write("# before EMES filtering %i positive sites: mode %s, P>%5.2f\n" % (len(total_sites), selection_mode, 0.5) )
        
        # filter according to emes: maximum significance larger than 0.9
        total_sites = set( filter( lambda x: max_per_site[x] > 0.9, total_sites) )

        if options.loglevel >= 2:
            options.stdlog.write("# after EMES filtering %i positive sites: mode %s, P>%5.2f\n" % (len(total_sites), selection_mode, 0.9) )

    else:
        if options.loglevel >= 2:
            options.stdlog.write("# extracted %i positive sites: mode %s, P>%5.2f\n" % (len(total_sites), selection_mode, options.filter_probabiltiy) )

    if mali and options.filter_mali:
        if options.filter_mali == "gaps":
            nfiltered = 0
            mali_length = mali.getLength()

            column_data = map(lambda x: Mali.MaliData( x, gap_chars = "Nn", mask_chars = "-."), mali.getColumns())
            new_sites = set()

            for x in total_sites:

                ## PAML uses one-based coordinates
                column = column_data[x-1]

                if column.mNChars != mali_length:
                    nfiltered += 1
                    if options.loglevel >= 3:
                        options.stdlog.write("# rejected position %i due to mali\n" % x )
                    continue

                new_sites.add( x )

            total_sites = new_sites

        if options.loglevel >= 2:
            options.stdlog.write("# after MALI filtering %i positive sites\n" % (len(total_sites) ))

    return total_sites, max_per_site

def mapSites2Codons( total_sites, max_per_site ):
    """map sites to codons and return codons
    """
    
    max_per_codon = {}
    total_codons = set()
        
    for site in total_sites:
        codon, pos = divmod( site - 1, 3 )
        codon += 1
        total_codons.add( codon )
        if codon not in max_per_codon:
            max_per_codon[codon] = 0
        max_per_codon[codon] = max(max_per_codon[codon], max_per_site[site] )
            
    return total_codons, max_per_codon

def convertMali2Mali( mali ):
    """convert a mali to a profile."""

    new_mali = alignlib.makeMultipleAlignment()
    for id in mali.getIdentifiers():
        s = alignlib.makeAlignatumFromString( mali[id] )
        s.thisown = 0
        new_mali.addAlignatum( s )

    return new_mali

if __name__ == "__main__":
    
    parser = E.OptionParser( version = "%prog version: $Id: codemls2tsv.py 2781 2009-09-10 11:33:14Z andreas $" )

    parser.add_option("--methods", dest="methods", type="choice", action="append",
                      choices=("summary-numbers", "jalview",
                               "positive-site-table", "positive-site-list",
                               "count-positive-sites" ),
                      help="methods for analysis."  )

    parser.add_option("--selection-mode", dest="selection_mode", type="choice", 
                      choices=("all", "consistent", "emes" ),
                      help="how to select positive sites."  )

    parser.add_option("--prefix", dest="prefix", type="string",
                      help="prefix for rows.")

    parser.add_option("--pattern-input-filenames", dest="pattern_input_filenames", type="string",
                      help="input pattern."  )

    parser.add_option("--filter-probability", dest="filter_probability", type="float",
                      help="threshold for probability above which to include positive sites [default=%default]."  )

    parser.add_option("--filter-omega", dest="filter_omega", type="float",
                      help="threshold for omega above which to include positive sites [default=%default]."  )

    parser.add_option("--models", dest="models", type="string",
                      help="restrict output to set of site specific models."  )

    parser.add_option("--analysis", dest="analysis", type="string",
                      help="restrict output to set of analysis [beb|neb]."  )

    parser.add_option("--significance-threshold", dest="significance_threshold", type="float",
                      help="significance threshold for log-likelihood test."  )

    parser.add_option("--filter-mali", dest="filter_mali", type="choice",
                      choices=("none", "gaps"),
                      help="filter by mali to remove gapped positions.")

    parser.add_option("--filename-mali", dest="filename_mali", type="string",
                      help="filename with multiple alignment used for calculating sites - used for filtering" )

    parser.add_option("--filename-map-mali", dest="filename_map_mali", type="string",
                      help="filename with multiple alignment to map sites onto." )

    parser.add_option("--jalview-titles", dest="jalview_titles", type="string",
                      help="comma separated list of jalview annotation titles." )

    parser.add_option("--jalview-symbol", dest="jalview_symbol", type="string",
                      help="symbol to use in jalview." )

    parser.set_defaults(
        methods = [],
        prefix = None,
        filter_probability = 0,
        filter_omega = 0,
        models = "",
        analysis = "",
        significance_threshold = 0.05,
        selection_mode = "consistent",
        filename_mali = None,
        filename_map_mali = None,
        jalview_symbol = "*",
        jalview_titles = "",
        filter_mali = None,
        )

    (options, args) = E.Start( parser )

    if options.jalview_titles:
        options.jalview_titles = options.jalview_titles.split(",")
    else:
        options.jalview_titles = args    
        
    options.models = options.models.split(",")
    options.analysis = options.analysis.split(",")
    
    for a in options.analysis:
        if a not in ("beb", "neb"):
            raise "unknown analysis section: '%s', possible values are 'beb' and/or 'neb'" % a

    for a in options.models:
        if a not in ("8", "2", "3"):
            raise "unknown model: '%s', possible values are 2, 3, 8" % a
    
    codeml = WrapperCodeML.CodeMLSites()

    ## filter and extract functions
    filter_f = lambda x: x.mProbability >= options.filter_probability and x.mOmega >= options.filter_omega
    extract_f = lambda x: x.mResidue

    ## read multiple results
    results = []
    ninput, noutput, nskipped = 0, 0, 0

    headers = []
    for f in args:
        ninput += 1
        try:
            results.append(codeml.parseOutput( open(f, "r").readlines() ))
        except WrapperCodeML.UsageError:
            if options.loglevel >= 1:
                options.stdlog.write("# no input from %s\n" % f)
            nskipped += 1
            continue
        noutput += 1
        headers.append( f )
        
    ## map of nested model (key) to more general model
    map_nested_models = { '8' : '7',
                          '2' : '1',
                          '3' : '0' }

    if options.filename_mali:
        mali = Mali.Mali()
        mali.readFromFile( open(options.filename_mali, "r") )
    else:
        mali = None


    ###############################################################
    ###############################################################
    ###############################################################
    ## use multiple alignment to map residues to a reference mali
    ## or a sequence.
    ###############################################################        
    if options.filename_map_mali:
        
        if not mali:
            raise "please supply the input multiple alignment, if residues are to be mapped."

        ## translate the alignments
        def translate( s ):
            sequence = s.mString
            seq = []
            for codon in [ sequence[x:x+3] for x in range(0, len(sequence), 3) ]:
                aa = Genomics.MapCodon2AA( codon )
                seq.append( aa )

            s.mString = "".join(seq)


        tmali = Mali.Mali()
        tmali.readFromFile( open(options.filename_mali, "r" ) )
        tmali.apply( translate )
        
        tmap_mali = Mali.Mali()
        tmap_mali.readFromFile( open(options.filename_map_mali, "r") )

        if tmap_mali.getAlphabet() == "na":
            tmap_mali.apply( translate )
        
        map_old2new = alignlib.makeAlignmentVector()

        mali1 = alignlib.makeProfileFromMali( convertMali2Mali( tmali ) )

        if tmap_mali.getLength() == 1:
            
            s = tmap_mali.values()[0].mString
            mali2 = alignlib.makeSequence( s )
            ## see if you can find an identical subsequence and then align to thisD
            for x in tmali.values():
                if s in re.sub( "[- .]+", "", x.mString):
                    mali1 = alignlib.makeSequence( x.mString )
                    break
        else:
            mali2 = alignlib.makeProfileFromMali( convertMali2Mali( tmap_mali ) )        

        alignator = alignlib.makeAlignatorDPFull( alignlib.ALIGNMENT_LOCAL, -10.0, -2.0 )
        alignator.align( map_old2new, mali1, mali2 )

        consensus = tmap_mali.getConsensus()
        
        if options.loglevel >= 4:
            options.stdlog.write( "# alphabet: %s\n" % tmap_mali.getAlphabet() )
            options.stdlog.write( "# orig  : %s\n" % tmali.getConsensus() )
            options.stdlog.write( "# mapped: %s\n" % consensus )
            options.stdlog.write( "# alignment: %s\n" % map_old2new.Write())
    else:
        map_old2new = None

    for method in options.methods:
        
        if method == "summary-numbers":

            options.stdlog.write( \
"""# Numbers of positive sites.
#
# The consistent row/column contains positive sites that are significant
# (above thresholds for probability and omega) for all models/analysis
# that have been selected (label: cons).
#
# The log-likelihood ratio test is performed for model pairs, depending
# on the output chosen.
# Significance threshold: %6.4f
# The pairs are 8 versus 7 and 2 versus 1 and 3 versus 0.
#
""" % options.significance_threshold )
            
            ## write header
            if options.prefix: options.stdout.write("prefix\t")
            
            options.stdout.write("method\tnseq\t" )
            h = []
            for model in options.models:
                for analysis in options.analysis:
                    h.append("%s%s" % (analysis, model) )
                h.append("p%s" % (model))
                h.append("df%s" % (model))
                h.append("chi%s" % (model))
                h.append("lrt%s" % (model))
                
            options.stdout.write("\t".join(h) )
            options.stdout.write("\tcons\tpassed\tfilename\n")

            nmethod = 0

            consistent_cols = [ None for x in range( len(options.analysis) ) ]
            passed_tests = {}
            for m in options.models: passed_tests[m] = 0
            
            for result in results:

                row_consistent = None

                if options.prefix:
                    options.stdout.write("%s" % (options.prefix ))
                
                options.stdout.write("%i" % nmethod)
                options.stdout.write("\t%i" % (result.mNumSequences ))
                                         
                npassed = 0
                
                for model in options.models:

                    sites = result.mSites[model]

                    ## do significance test
                    full_model, null_model = model, map_nested_models[model]
                    
                    lrt = Stats.doLogLikelihoodTest(
                        result.mSites[full_model].mLogLikelihood, 
                        result.mSites[full_model].mNumParameters, 
                        result.mSites[null_model].mLogLikelihood, 
                        result.mSites[null_model].mNumParameters, 
                        options.significance_threshold )

                    x = 0
                    for analysis in options.analysis:
                        
                        if analysis == "neb":
                            s = set(map( extract_f, filter( filter_f, sites.mNEB.mPositiveSites)))
                            
                        elif analysis == "beb":
                            s = set(map( extract_f, filter( filter_f, sites.mBEB.mPositiveSites)))                            
                            
                        options.stdout.write("\t%i" % ( len(s) ) )

                        if not lrt.mPassed:
                            s = set()
                            
                        if row_consistent == None:
                            row_consistent = s
                        else:
                            row_consistent = row_consistent.intersection( s )

                        if consistent_cols[x] == None:
                            consistent_cols[x] = s
                        else:
                            consistent_cols[x] = consistent_cols[x].intersection(s)

                        x += 1

                    if lrt.mPassed:
                        c = "passed"
                        passed_tests[model] += 1
                        npassed += 1
                    else:
                        c = "failed"
                        
                    options.stdout.write("\t%5.2e\t%i\t%5.2f\t%s" %\
                                         (lrt.mProbability, 
                                          lrt.mDegreesFreedom, 
                                          lrt.mChiSquaredValue, 
                                          c))
                    
                options.stdout.write("\t%i\t%i\t%s\n" % (len(row_consistent), npassed, headers[nmethod]))
                
                nmethod += 1
                
            if options.prefix:
                options.stdout.write("%s\t" % options.prefix )

            options.stdout.write("cons" )
            
            row_consistent = None
            total_passed = 0
            for model in options.models:

                x = 0
                
                for analysis in options.analysis:
                    
                    s = consistent_cols[x]
                    if s == None:
                        s = set()
                        
                    options.stdout.write("\t%i" % (len(s)))

                    if row_consistent == None:
                        row_consistent = s 
                    else:
                        row_consistent = row_consistent.intersection( s )

                    x += 1
                    
                options.stdout.write("\tna\t%i" % passed_tests[model] )
                total_passed += passed_tests[model]
                
            options.stdout.write("\t%i\t%i\n" % (len(row_consistent), total_passed) )

        elif method == "jalview":

            options.stdout.write("JALVIEW_ANNOTATION\n" )
            options.stdout.write("# Created: %s\n\n" % (time.asctime(time.localtime(time.time()))))

            l = 1
            x = 0
            for result in results:
                
                sites, significance = selectPositiveSites( [result], options.selection_mode, options, mali )

                codes = [""] * result.mLength

                if len(sites) == 0: continue

                for site in sites:
                    codes[site-1] = options.jalview_symbol

                options.stdout.write("NO_GRAPH\t%s\t%s\n" % (options.jalview_titles[x], "|".join( codes ) ))
                x += 1
                
        elif method == "count-positive-sites":

            sites, significance = selectPositiveSites( results, options.selection_mode, options, mali )

            options.stdout.write( "%i\n" % (len(sites) ))
            
        elif method in ( "positive-site-table", ):

            sites, significance = selectPositiveSites( results, options.selection_mode, options, mali )

            headers = ["site", "P"]
            if map_old2new:
                headers.append( "mapped" )
                headers.append( "Pm" )                
                
            options.stdout.write( "\t".join( headers ) + "\n" )

            sites = list(sites)
            sites.sort()
            nmapped, nunmapped = 0, 0
            for site in sites:
                values = [site, "%6.4f" % significance[site]]

                if map_old2new:
                    r = map_old2new.mapRowToCol(site)
                    if r == 0:
                        values.append("na")
                        values.append("")
                        nunmapped += 1
                        if options.loglevel >= 2:
                            options.stdlog.write("# unmapped residue: %i\n" % site )
                    else:
                        values.append(r)
                        values.append(consensus[r-1])
                        nmapped += 1
                        
                options.stdout.write( "\t".join( map(str, (values ))) + "\n" )

            if options.loglevel >= 1:
                options.stdlog.write("# sites: ninput=%i, noutput=%i, nskipped=%i\n" % (len(sites), nmapped, nunmapped))
                
    if options.loglevel >= 1:
        options.stdlog.write("# ninput=%i, noutput=%i, nskipped=%i\n" % (ninput, noutput, nskipped))
    
    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
cgat_logfiles2tsv.py - create summary from logfiles
===================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This scripts are collection of logfiles and collates
summary information.

This script uses the ``# job finished`` tag that
is added by scripts using the module :mod:`Experiment`.

Usage
-----

Example::

   python cgat_logfiles2tsv.py --help

Type::

   python cgat_logfiles2tsv.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import tempfile
import subprocess
import optparse
import gzip
import glob

from types import *

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
import CGAT.Logfile as Logfile

if __name__ == "__main__":
    
    parser = E.OptionParser( version = "%prog version: $Id: cgat_logfiles2tsv.py 2781 2009-09-10 11:33:14Z andreas $" )

    parser.add_option( "-g", "--glob", dest="glob_pattern", type="string" ,
                       help="glob pattern to use for collecting files [%default].")

    parser.add_option( "-f", "--file-pattern", dest="file_pattern", type="string",
                       help="only check files matching this pattern [%default]." )

    parser.add_option( "-m", "--mode", dest="mode", type="choice",
                       choices = ("file", "node" ),
                       help="analysis mode [%default]." )

    parser.add_option( "-r", "--recursive", action="store_true",
                       help="recursively look for logfiles from current directory [%default]." )

    parser.set_defaults(
        truncate_sites_list = 0,
        glob_pattern = "*.log",
        mode = "file",
        recursive = False,
        )

    (options, args) = E.Start( parser )

    if args:
        filenames = args
    elif options.glob_pattern:
        filenames = glob.glob( options.glob_pattern )

    if len(filenames) == 0:
        raise "no files to analyse"

    if options.mode == "file":
        totals = Logfile.LogFileData()

        options.stdout.write( "file\t%s\n" % totals.getHeader() )

        for filename in filenames:
            if filename == "-":
                infile = sys.stdin
            elif filename[-3:] == ".gz":
                infile = gzip.open( filename, "r" )
            else:
                infile = open(filename, "r" )

            subtotals = Logfile.LogFileData()
            for line in infile:
                subtotals.add( line )

            infile.close()

            options.stdout.write( "%s\t%s\n" % (filename, str(subtotals) ) )
            totals += subtotals

        options.stdout.write( "%s\t%s\n" % ("total", str(totals) ) )

    elif options.mode == "node":
        
        chunks_per_node = {}

        rx_node = re.compile( "# job started at .* \d+ on (\S+)" )

        for filename in filenames:
            if filename == "-":
                infile = sys.stdin
            elif filename[-3:] == ".gz":
                infile = gzip.open( filename, "r" )
            else:
                infile = open(filename, "r" )
                
            data = Logfile.LogFileDataLines()
               
            for line in infile:
                
                if rx_node.match(line):
                    node_id = rx_node.match(line).groups()[0]
                    data = Logfile.LogFileDataLines()
                    if node_id not in chunks_per_node:
                        chunks_per_node[node_id] = []
                    chunks_per_node[node_id].append( data )
                    continue

                data.add( line )

        options.stdout.write( "node\t%s\n" % data.getHeader() )
        total = Logfile.LogFileDataLines()
                   
        for node, data in sorted(chunks_per_node.items()):
            subtotal = Logfile.LogFileDataLines()
            for d in data:
                # options.stdout.write( "%s\t%s\n" % (node, str(d) ) )  
                subtotal += d

            options.stdout.write( "%s\t%s\n" % (node, str(subtotal) ) )  

            total += subtotal
            
        options.stdout.write( "%s\t%s\n" % ("total", str(total) ) )  

    E.Stop()
    



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
rates2rates.py - operate on rates
=================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

input = tab separated table ids, rates and G+C

This script corrects a substitution rate for mutational
biases. The mutational bias is estimated from a given
distribution of rates and G+C content in neutrally 
evolving regions, for example ancestral repeats.

Usage
-----

Example::

   python rates2rates.py --help

Type::

   python rates2rates.py --help

for command line help.

Documentation
-------------

Code
----

'''
import os
import sys
import string
import re
import getopt
import time
import optparse
import math
import tempfile
import CGAT.Experiment as E
import CGAT.CSV as CSV
import CGAT.IOTools as IOTools
import CGAT.Stats as Stats
import numpy

def readRates( infile ):
    """read rates and G+C from a tab-separated file."""
    from rpy import r as R
    import rpy

    handle, name = tempfile.mkstemp()
    os.close(handle)
    outfile = open(name, "w")

    first = True
    headers = []
    for line in infile:
        if line[0] == "#": continue
        data = line[:-1].split("\t")
        if first:
            headers = data
            first = False
            continue

        outfile.write( line )
    outfile.close()

    assert len(headers) == 3, "malformatted file of rates, please supply id, g+c, rate"

    rpy.set_default_mode(rpy.NO_CONVERSION)
    matrix = R.read_table( name, na_string = ("NA", "na"), col_names=headers ) 
    rpy.set_default_mode(rpy.BASIC_CONVERSION)
    os.remove( name )
    return matrix, headers

def main():

    parser = E.OptionParser( version = "%prog version: $Id: rates2rates.py 2781 2009-09-10 11:33:14Z andreas $", usage = globals()["__doc__"])

    parser.add_option( "--output-filename-pattern", dest="output_filename_pattern", type="string",
                      help="pattern for additional output files [%default]."  )

    parser.add_option( "--input-filename-neutral", dest="input_filename_neutral", type="string",
                      help="a tab-separated file with rates and G+C content in neutrally evolving regions [%default]."  )

    parser.set_defaults(
        input_filename_neutral = None,
        output_filename_pattern = "%s",
        normalize = True,
        hardcopy = None,
        )

    (options, args) = E.Start( parser, add_csv_options = True )

    if not options.input_filename_neutral:
        raise ValueError( "please supply a file with neutral rates." )

    lines = options.stdin.readlines()
    if len(lines) == 0:
        raise IOError ( "no input" )

    from rpy import r as R
    import rpy

    R.png( options.output_filename_pattern % "fit" + ".png", width=1024, height=768, type="cairo")
    matrix, headers = readRates( open( options.input_filename_neutral, "r" ) )
    R.assign("matrix", matrix)
    R.assign("headers", headers)
    nref = R( """length( matrix[,1] )""" )

    dat = R("""dat <- data.frame(x = matrix[,2], y = matrix[,3])""")
    mod = R("""mod <- lm( y ~ x, dat)""")

    R("""plot( matrix[,2], matrix[,3], cex=%s, col="blue", pch="o", xlab="%s", ylab="%s" %s)""" % (0.3, headers[1], headers[2], "") )
    R("""new <- data.frame(x = seq( min(matrix[,2]), max(matrix[,2]), (max(matrix[,2]) - min(matrix[,2])) / 100))""")
    R("""predict(mod, new, se.fit = TRUE)""")
    R("""pred.w.plim <- predict(mod, new, interval="prediction")""")
    R("""pred.w.clim <- predict(mod, new, interval="confidence")""")
    R("""matpoints(new$x,cbind(pred.w.clim, pred.w.plim[,-1]), lty=c(1,2,2,3,3), type="l")""")
    R.mtext(
        "y = %f * x + %f, r=%6.4f, n=%i" % (mod["coefficients"]["x"], 
                                            mod["coefficients"]["(Intercept)"], 
                                            R("""cor( dat )[2]"""), 
                                            nref ),
        3,
        cex = 1.0)

    R("""mean_rate <- mean( matrix[,3] )""")

    data_matrix, data_headers = readRates( lines )
    R.assign("data_matrix", data_matrix)
    R.assign("data_headers", data_headers)
    ndata = R( """length( data_matrix[,1] )""" )
    
    R("""points( data_matrix[,2], data_matrix[,3], cex=%s, col="red", pch="o" %s)""" % (0.3, "") )
    R("""topred <- data.frame( x = data_matrix[,2] )""")
    R("""corrected_rates <- predict( mod, topred, se.fit = TRUE )""")
    uncorrected = R("""uncorrected <- data_matrix[,3] / mean_rate """) 
    corrected = R("""corrected <- as.vector(data_matrix[,3] / corrected_rates$fit)""")
    R.dev_off()
    
    R.png( options.output_filename_pattern % "correction" + ".png", width=1024, height=768, type="cairo")
    R("""plot( uncorrected, corrected, cex=%s, col="blue", pch="o", xlab="uncorrected rate", ylab="corrected rate" %s)""" % (0.3, "") )
    R.dev_off()

    E.Stop()

if __name__ == "__main__":
    sys.exit( main() )



"""
RateEstimation.py - utilities for computing rate estimates for codon models.
=============================================================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

"""

import Bio.Data.CodonTable
try:
    from XGram.Generator.Prebuilt import Codons
    from XGram.Model import Annotation
    import XGram.Exceptions
except ImportError:
    pass

import Genomics
##-----------------------------------------------------------------------------------
##-----------------------------------------------------------------------------------
##-----------------------------------------------------------------------------------
def evaluateCodonPair( codon1, codon2 ):
    """evaluate differences between codon pair."""
    
    codon_table = Bio.Data.CodonTable.standard_dna_table

    changes = []
    for x in range(0,3):
        if codon1[x] != codon2[x]:
            changes.append( (x, codon1[x], codon2[x]) )

    if len(changes) != 1:
        return False, False, False

    change = ( changes[0][1], changes[0][2] )

    is_synonymous = codon_table.forward_table[codon1] == codon_table.forward_table[codon2]
    is_transition = change in (("A","G"), ("G","A"), ("T","C"), ("C","T"))

    return True, is_synonymous, is_transition

##-------------------------------------------------------------------
def countSubstitutions( pi, Q ):
    """count substitituions given a matrix Q and frequencies pi."""
    
    rI, rV, rS, rN = 0, 0, 0, 0

    codon_table = Bio.Data.CodonTable.standard_dna_table
    codons = codon_table.forward_table.keys()

    for codon_i in codons:
        for codon_j in codons:
            if codon_i == codon_j: continue

            is_single, is_synonymous, is_transition = evaluateCodonPair( codon_i, codon_j)

            v = Q[codon_i][codon_j] * pi[codon_i]

            if is_transition:
                rI += v
            else:
                rV += v

            if is_synonymous:
                rS += v
            else:
                rN += v

    return rI, rV, rS, rN

##-------------------------------------------------------------------
def initializeQMatrix( codons ):
    """get an initialized Q matrix."""
    
    Q = {}
    for codon_i in codons:
        Q[codon_i] = {}            
        for codon_j in codons:
            Q[codon_i][codon_j] = 0.0
            
    return Q

##-------------------------------------------------------------------
def getQMatrix( pi, Rsi, Rsv, Rni, Rnv):
    """build a q matrix.

    Diagonal elements are set to the negative of the row sums.
    The matrix is normalized such that trace of the matrix is -1.
    """

    codons = Bio.Data.CodonTable.standard_dna_table.forward_table.keys()

    Q = initializeQMatrix( codons )

    trace = 0.0
    for codon_i in codons:
        row_sum = 0.0
        for codon_j in codons:
            if codon_i == codon_j: continue

            is_single, is_synonymous, is_transition = evaluateCodonPair( codon_i, codon_j )

            if not is_single: continue

            if is_synonymous:
                if is_transition:
                    v = Rsi
                else:
                    v = Rsv
            else:
                if is_transition:
                    v = Rni
                else:
                    v = Rnv

            v *= pi[codon_j]
            Q[codon_i][codon_j] = v
            row_sum += v

        Q[codon_i][codon_i] = -row_sum
        trace += pi[codon_i] * row_sum

    for codon_i in codons:
        for codon_j in codons:
            Q[codon_i][codon_j] /= trace

    return Q, trace

##--------------------------------------------------------------
def getRateMatrix( trained_model, terminals = None ):
    """return a rate matrix from an xrate grammar.

    terminals: return rate matrix and frequencies for these
        terminals. If none are given, a dictionaries of
        matrices and frequencies are returned.

                
    """

    if terminals:
        xt = (terminals,)
    else:
        xt = trained_model.mGrammar.getTerminals()
        pis, matrices = {}, {}

    for tt in xt:
        ## retrieve the terminal frequencies for all codons
        xpi = trained_model.evaluateTerminalFrequencies()[tt]

        pi = {}
        for codon, f in xpi.items():
            pi["".join(codon).upper()] = f

        ## retrieve the rate matrix for all codons
        xmatrix = trained_model.evaluateRateMatrix()[tt]
        matrix = {}
        for codon1, v in xmatrix.items():
            x = {}
            for codon2, f in xmatrix[codon1].items():
                x["".join(codon2).upper()] = v
            matrix["".join(codon1).upper()] = x

        if terminals:
            return pi, matrix
        else:
            pis[tt] = pi
            matrices[tt] = matrix
    
    return pis, matrices

##-----------------------------------------------------------------------------------
##-----------------------------------------------------------------------------------
##-----------------------------------------------------------------------------------
def setFrequencies( model, mali, prefix = ""):
    """set frequencies in a model according to those observed in data.

    prefix: prefix for rate parameters. 

    Frequencies are labelled:
    pa0, pc0, ..., pa1, pc1, ..., pa2, pc2, ...
    """
    
    try:
        frequencies = Codons.getFrequenciesPerCodonPosition( [ x.mString for x in mali.values() ] )
    except XGram.Exceptions.UsageError:
        return

    ## build a dummy grammar to insert frequencies
    dummy_grammar = XGram.Model.Grammar()
    for x in range(0,3):
        params = []
        for a in ('A', 'C', 'G', 'T'):
            params.append( ("%sp%s%i" % (prefix, a.lower(), x), frequencies[x][a]) )
        dummy_grammar.addVariable( params )
        
    model.mGrammar.copyParameters( dummy_grammar, 
                                   ignore_missing = True)

def getDistanceGTR( pi, matrix):
    """obtain distance from a GTR model.
    see Felsenstein 1994, pp 209
    """
    alphabet = pi.keys()
    Q = initializeQMatrix( alphabet )
    
    # fill matrix AtD and compute trace
    trace = 0
    for i in alphabet:
        row_sum = 0
        for j in alphabet:
            if i == j: continue
            v = pi[i] * matrix[i][j]
            Q[i][j] = v
            row_sum += v
        
        Q[i][i] = -row_sum
        trace += row_sum
        
    for i in alphabet:
        for j in alphabet:
            Q[i][j] /= trace

    return Q, trace



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id$
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
submit.py - run a collection of qsub scripts on the cluster
===========================================================

:Author: Andreas Heger
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script will submit a collection of qsub scripts on the cluster.
The script will only submit those jobs for which a result is not 
present.

Usage
-----

Example::

   python submit.py *.qsub

Type::

   python submit.py --help

for command line help.

Documentation
-------------

Code
----

'''
USAGE="""python submit.py [OPTIONS] 

submit a list of jobs to the cluster.
"""

import os
import sys
import re
import string
import optparse
import time
import glob
import subprocess
import stat

import CGAT.Experiment as E
import CGAT.IOTools as IOTools

def checkPythonRuns( filename ):
    """returns true if a python run is complete."""
    last_line = IOTools.getLastLine( filename )
    return re.match( "# job finished", last_line )
    
def isNewer( a, b ):
    """return true if file a is newer than file b."""

    ## get times of most recent access
    at = os.stat( a )[7]
    bt = os.stat( b )[7]

    return at > bt

if __name__ == '__main__':

    parser = E.OptionParser( version = "%prog version: $Id: submit.py 2782 2009-09-10 11:40:29Z andreas $", usage = globals()["__doc__"] )

    parser.add_option( "-g", "--glob", dest="glob_pattern", type="string" ,
                       help="glob pattern to use for collecting cluster jobs descriptions [%default]")

    parser.add_option( "-i", "--input-pattern", dest="input_pattern", type="string" ,
                       help="regular expression to extract job id from filename [%default].")

    parser.add_option( "-o", "--output-pattern", dest="output_pattern", type="string" ,
                       help="string to convert a job id to a filename [%default].")


    parser.set_defaults( glob_pattern = "job*.qsub",
                         input_pattern = "(\S+).qsub",
                         output_pattern = "%s.stdout",
                         remove_old = True,
                         force = False,
                         check_completeness = "python",
                         )

    (options, args) = E.Start( parser, 
                                        add_pipe_options = True )

    if args:
        filenames = args
    elif options.glob_pattern:
        filenames = glob.glob( options.glob_pattern )
        
    ninput, nrun, nskipped, nerrors = 0, 0, 0, 0
    ndeleted = 0

    if options.check_completeness == "python":
        isComplete = checkPythonRuns

    ##############################################################
    ##############################################################
    ##############################################################
    ## decide what to do
    ##############################################################
    jobs = []
    files_to_delete = []

    for filename in filenames:
        
        ninput += 1
        try:
            job_name = re.search( options.input_pattern, filename ).groups()[0]
        except AttributeError:
            options.stderr.write( "# could not extract invariant job name from %s\n" % filename )
            nerrors += 1
            continue
        
        result_filename = options.output_pattern % job_name
        
        do = False
        status = "up-to-date"

        if options.force:
            status = "force"
            do = True
            
        if not do:
            if os.path.exists( result_filename ):
                if isNewer( filename, result_filename ):
                    status = "newer"
                    do = True
                    if options.remove_old:
                        files_to_delete.append( result_filename )
                if not do and not isComplete( result_filename ):
                    status = "incomplete"
                    do = True
                    if options.remove_old:
                        files_to_delete.append( result_filename )
            else:
                status = "missing"
                do = True

        if options.loglevel >= 1:
            options.stdlog.write("# %s->%s (%s)\n" % (filename, result_filename, status))
                
        if not do:
            nskipped += 1
            continue

        jobs.append( filename )

    ##############################################################
    ##############################################################
    ##############################################################
    ## delete old files
    ##############################################################
    for filename in files_to_delete:
        if os.path.exists( filename ):
            os.remove( filename )
            ndeleted += 1

    ##############################################################
    ##############################################################
    ##############################################################
    ## start jobs
    ##############################################################
    for filename in jobs:

        cmd = "qsub %s" % filename
        try:
            retcode = subprocess.call(cmd, shell=True)
            if retcode != 0:
                if options.loglevel >= 1:
                    options.stdlog.write("# ERROR: failed to execute %s\n" % cmd)
                nerrors += 1
                continue
        except OSError, e:
            if options.loglevel >= 1:
                options.stdlog.write("# ERROR: failed to execute %s with msg %s\n" % (cmd, e))
        nrun += 1

    if options.loglevel >= 1:
        options.stdlog.write("# ninput=%i, nrun=%i, nskipped=%i, ndeleted=%i, nerrors=%i\n" %\
                                 (ninput, nrun, nskipped, ndeleted, nerrors ) )

    E.Stop()



################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
cgat_script_template.py
=============================================

:Author: 
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

Usage
-----

Example::

   python cgat_script_template.py --help

Type::

   python cgat_script_template.py --help

for command line help.

Documentation
-------------

taxid:
    human: 9606
    mouse: 10090

Controlled vocabularies:

library_type:

    GENOMIC (Genomic DNA (includes PCR products from genomic DNA))
    TRANSCRIPTOMIC (Transcription products or non genomic DNA (EST, cDNA, RT-PCR, 
                    screened libraries))
    METAGENOMIC (Mixed material from metagenome)
    METATRANSCRIPTOMIC (Transcription products from community targets)
    SYNTHETIC (Synthetic DNA)
    VIRAL RNA (Viral RNA)
    OTHER (Other, unspecified, or unknown library source material)

library_selection:

    RANDOM (Random selection by shearing or other method)
    PCR (Source material was selected by designed primers)
    RANDOM PCR (Source material was selected by randomly generated primers)
    RT-PCR (Source material was selected by reverse transcription PCR)
    HMPR (Hypo-methylated partial restriction digest)
    MF (Methyl Filtrated)
    repeat fractionation (replaces: CF-S, CF-M, CF-H, CF-T)
    size fractionation
    MSLL (Methylation Spanning Linking Library)
    cDNA (complementary DNA)
    ChIP (Chromatin immunoprecipitation)
    MNase (Micrococcal Nuclease (MNase) digestion)
    DNAse (Deoxyribonuclease (MNase) digestion)
    Hybrid Selection (Selection by hybridization in array or solution)
    Reduced Representation (Reproducible genomic subsets, often generated by 
                            restriction fragment size selection, containing a 
                            manageable number of loci to facilitate re-sampling)
    Restriction Digest (DNA fractionation using restriction enzymes)
    5-methylcytidine antibody (Selection of methylated DNA fragments using an 
                antibody raised against 5-methylcytosine or 5-methylcytidine (m5C))
    MBD2 protein methyl-CpG binding domain (Enrichment by methyl-CpG binding domain)
    CAGE (Cap-analysis gene expression)
    RACE (Rapid Amplification of cDNA Ends)
    MDA (Multiple displacement amplification)
    padlock probes capture method (to be used in conjuction with Bisulfite-Seq)
    other (Other library enrichment, screening, or selection process)
    unspecified (Library enrichment, screening, or selection is not specified)

Code
----

'''

import os
import sys
import re
import optparse
import urllib2
import urllib
import collections
import xml.etree.ElementTree as ET

import CGAT.Experiment as E
import CGAT.IOTools as IOTools
from Bio import Entrez

# mapping various combinations of library_source and selection
# to an experiment type
MAP_CODE2DESIGN = \
    {
    ("CAGE", "TRANSCRIPTOMIC"): 'CAGE' ,
    ("ChIP", "GENOMIC"): "ChIP-Seq",
    ("DNase", "GENOMIC"): "DNase-Seq",
    ("Hybrid Selection", "GENOMIC"): "Exome-Seq",
    ("cDNA", "TRANSCRIPTOMIC"): "RNA-Seq",
    ("PCR", "GENOMIC"): "Genome-Seq",
    }

# all other:
# ChIP    OTHER   45
# ChIP    TRANSCRIPTOMIC  17
# DNase   TRANSCRIPTOMIC  133
# "Hybrid Selection"      TRANSCRIPTOMIC  8
# "MBD2 protein methyl-CpG binding domain"        GENOMIC 238
# MDA     GENOMIC 340
# MNase   GENOMIC 316
# PCR     GENOMIC 1829
# PCR     METAGENOMIC     56
# PCR     OTHER   12
# PCR     SYNTHETIC       8
# PCR     TRANSCRIPTOMIC  73
# RACE    TRANSCRIPTOMIC  4
# RANDOM  GENOMIC 51129
# RANDOM  METAGENOMIC     158
# RANDOM  METATRANSCRIPTOMIC      2
# RANDOM  "NON GENOMIC"   37
# RANDOM  OTHER   55
# RANDOM  TRANSCRIPTOMIC  751
# "RANDOM PCR"    GENOMIC 172
# "RANDOM PCR"    TRANSCRIPTOMIC  41
# RT-PCR  GENOMIC 3
# RT-PCR  "NON GENOMIC"   8
# RT-PCR  TRANSCRIPTOMIC  126
# "Reduced Representation"        GENOMIC 442
# "Restriction Digest"    GENOMIC 87
# cDNA    GENOMIC 63
# cDNA    METATRANSCRIPTOMIC      2
# cDNA    "NON GENOMIC"   73
# cDNA    OTHER   431
# other   GENOMIC 2845
# other   METAGENOMIC     16
# other   OTHER   306
# other   SYNTHETIC       2756
# other   TRANSCRIPTOMIC  428
# "padlock probes capture method" GENOMIC 61
# "size fractionation"    GENOMIC 121
# "size fractionation"    METAGENOMIC     8
# "size fractionation"    OTHER   2
# "size fractionation"    TRANSCRIPTOMIC  1303
# unspecified     GENOMIC 5138
# unspecified     OTHER   50
# unspecified     SYNTHETIC       14
# unspecified     TRANSCRIPTOMIC  337

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option( "--library-source", dest="library_source", type="string",
                      help="supply help"  )

    parser.add_option( "--library-selection", dest="library_selection", type="string",
                      help="supply help"  )

    parser.add_option( "--tax-id", dest="tax_id", type="int",
                      help="supply help"  )

    parser.set_defaults( library_source = None,
                         library_selection = None,
                         tax_id = 9606 )

    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    # tree = ET.parse('/ifs/home/andreas/ena.xml')
    # root = tree.getroot()

    # for study in root.findall("STUDY"):
    #     alias = study.attrib["alias"]
    #     center_name = study.attrib["center_name"]
    #     accession   = study.attrib["accession"]
    #     try:
    #         description = study.find("*/STUDY_DESCRIPTION").text
    #         description = description.encode('ascii', 'ignore')
    #     except AttributeError:
    #         description = ""

    #     options.stdout.write( "\t".join( (alias, 
    #                                       accession,
    #                                       center_name,
    #                                       description ) ) + "\n")

    #query_url = "http://www.ebi.ac.uk/ena/data/warehouse/search?query=%22tax_eq%289606%29%20AND%20library_source=%22TRANSCRIPTOMIC%22%20AND%20%28instrument_model=%22Illumina%20Genome%20Analyzer%20II%22%20OR%20instrument_model=%22Illumina%20Genome%20Analyzer%22%20OR%20instrument_model=%22Illumina%20Genome%20Analyzer%20IIx%22%20OR%20instrument_model=%22Illumina%20HiScanSQ%22%20OR%20instrument_model=%22Illumina%20HiSeq%201000%22%20OR%20instrument_model=%22Illumina%20HiSeq%202000%22%20OR%20instrument_model=%22Illumina%20HiSeq%202500%22%29%22&domain=read&download=txt"
    #query_url = "http://www.ebi.ac.uk/ena/data/view/search?query=%22tax_eq%289606%29%20AND%20library_source=%22TRANSCRIPTOMIC%22%20AND%20%28instrument_model=%22Illumina%20Genome%20Analyzer%20II%22%20OR%20instrument_model=%22Illumina%20Genome%20Analyzer%22%20OR%20instrument_model=%22Illumina%20Genome%20Analyzer%20IIx%22%20OR%20instrument_model=%22Illumina%20HiScanSQ%22%20OR%20instrument_model=%22Illumina%20HiSeq%201000%22%20OR%20instrument_model=%22Illumina%20HiSeq%202000%22%20OR%20instrument_model=%22Illumina%20HiSeq%202500%22%29%22&domain=read&download=txt"
    #query_url = "http://www.ebi.ac.uk/ena/data/warehouse/search?query=%22(instrument_model=%22Illumina%20HiSeq%202000%22%20OR%20instrument_model=%22Illumina%20HiSeq%201000%22%20OR%20instrument_model=%22Illumina%20HiSeq%202500%22)%20AND%20library_layout=%22PAIRED%22%20AND%20library_source=%22TRANSCRIPTOMIC%22%22&domain=read"
    # query_url = "http://www.ebi.ac.uk/ena/data/view/A00145&display=xml"

    query_url = "http://www.ebi.ac.uk/ena/data/warehouse/search"
    data_url = "http://www.ebi.ac.uk/ena/data/view"

    #params = None
    # query_url = "http://www.ebi.ac.uk/ena/data/view/DRP000011&display=xml"

    fields = ['base_count', 
              'read_count', 
              'instrument_model', 
              'scientific_name', 
              'library_layout', 
              'library_source',
              'library_strategy', 
              'library_selection', 
              'experiment_accession', 
              'experiment_title', 
              'study_accession', 
              'study_title',
              'first_public',
              'submission_accession',
              'center_name',
              ]

    query = 'tax_eq(%i) AND instrument_platform="ILLUMINA"' % (options.tax_id)

    if options.library_source:
        query += ' AND library_source="%s" ' % options.library_source
    if options.library_selection:
        query += ' AND library_selection="%s" ' % options.library_selection

    # collect pre-study results
    params = urllib.urlencode( { 'query' :  query,
                                 'display' : 'report',
                                 'fields' : ",".join(fields),
                                 'result' : 'read_run' } )

    E.debug( "?".join( (query_url, params )) )

    lines = urllib2.urlopen( query_url, params )

    header = lines.readline()
    
    fields.insert( 0, 'run_accession' )

    DATA = collections.namedtuple( "DATA", fields )

    fields.append( "read_length" )
    fields.append( "design" )

    table_study = options.stdout # IOTools.openFile( "study.tsv", "w" )
    table_study.write( "\t".join( fields ) + "\n" )
    # collect a list of all studies
    studies = set()

    for line in lines:
        # line endings are \r\n for data, but only \n for header
        line = line[:-2]

        data = DATA( *line.split("\t") )
        try:
            read_length = float(data.base_count) / float(data.read_count)
        except ValueError: 
            read_length = 0

        if data.library_layout == "PAIRED":
            read_length /= 2.0

        design = MAP_CODE2DESIGN.get( 
            (data.library_selection, data.library_source),
            "other" )
            
        table_study.write( line + "\t" + str(read_length) + "\t" + design + "\n" )

        studies.add( data.study_accession )

    table_studies = IOTools.openFile( "studies.tsv", "w" )
    studies_fields = ["study_accession", "nreferences", "pubmed_ids" ]

    table_studies.write( "\t".join( studies_fields ) + "\n" )

    return

    # params = urllib.urlencode( { 'display' : 'xml' } )
    # url =  "/".join( ( data_url, 'SRP013999') ) + "&" + params
    # print urllib2.urlopen( url ).read()

    for study_accession in studies:
        # get additional info
        params = urllib.urlencode( { 'display' : 'xml' } )
        url =  "/".join( ( data_url, study_accession) ) + "&" + params

        info_lines = urllib2.urlopen( url )
        tree = ET.parse(info_lines)
        root = tree.getroot()

        pmids = []
        for link in root.findall('*//XREF_LINK' ):
            db = link.find('DB').text
            if db == "pubmed":
                pmids.append( link.find('ID').text )

        # get geo
        geos = []
        for attribute in root.findall('*//STUDY_ATTRIBUTE'):
            if attribute.find('TAG').text == "GEO Accession":
                geos.append( attribute.find('VALUE').text )
        
        params = { 'dbfrom' : 'gds',
                   'db' : 'pubmed',
                   }
        
        geo_pmids = []
        for geo in geos:
            Entrez.email = "andreas.heger@dpag.ox.ac.uk"
            handle = Entrez.esearch(db="gds", retmax=1, term=geo)
            record = Entrez.read(handle)
            
            uids = record['IdList']
            handle.close()
            
            for uid in uids:
                record = Entrez.read( Entrez.elink(dbfrom="gds", 
                                                   dbto = "pubmed",
                                                   id = uid ) )
                linksets = record[0]["LinkSetDb"]
                if not linksets: continue

                assert len(linksets) == 1
                for linksetdb in linksets:
                    geo_pmids = [ x['Id'] for x in linksetdb["Link"] ]

        if not pmids: pmids = geo_pmids 

        table_studies.write( "\t".join( map(str, ( 
                        study_accession,
                        len(pmids),
                        ",".join(pmids),
                        len(geos),
                        ",".join(geos)) )) + "\n" )
        

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )




import glob
import sys
import os

#######################################################################
## Check for dependencies
##
## Is there a way to do this more elegantly?
##     1. Run "pip install numpy"
##     2. Wrap inside functions (works for numpy/pysam, but not cython)
#######################################################################
#######################################################################
#######################################################################
try:
    import numpy
except ImportError:
    raise ImportError("the CGAT code collection requires numpy to be installed before running setup.py (pip install numpy)" )

try:
    import Cython
except ImportError:
    raise ImportError("the CGAT code collection requires cython to be installed before running setup.py (pip install cython)" )

try: 
    import pysam
except ImportError:
    raise ImportError("the CGAT code collection requires pysam to be installed before running setup.py (pip install pysam)" )

########################################################################
########################################################################
## Import setuptools
## Use existing setuptools
try:
    from setuptools import setup, find_packages, Extension
except ImportError:
    ## try to get via ez_setup
    ## ez_setup did not work on all machines tested as
    ## it uses curl with https protocol, which is not
    ## enabled in ScientificLinux
    import ez_setup
    ez_setup.use_setuptools()
    from setuptools import setup, find_packages, Extension

from Cython.Distutils import build_ext

###############################################################
###############################################################
# Perform a CGAT Code Collection Installation
INSTALL_CGAT = True

major, minor1, minor2, s, tmp = sys.version_info

if major==2:
    extra_dependencies = [ 'web.py>=0.37',
                           'xlwt>=0.7.4', 
                           'matplotlib-venn>=0.5' ]
elif major==3:
    extra_dependencies = []

if major==2 and minor1<7 or major<2:
    raise SystemExit("""CGAT requires Python 2.6 or later.""")

# Dependencies shared between python 2 and 3
shared_dependencies = [
    'sphinx>=1.0.5',
    'SphinxReport>=2.0',
    'rpy2>=2.3.4',
    'numpy>=1.7',
    'scipy>=0.11',
    'matplotlib>=1.2.1', 
    'sqlalchemy>=0.7.0', 
    'pysam>=0.7',
    'openpyxl>=1.5.7',
    'MySQL-python>1.2.3',
    'biopython>=1.61',
    'scipy>=0.7.0',
    # Out of date, install manually latest version
    # 'bx-python>=0.7.1',
    'networkx>=1.8.1',
    'PyGreSQL>=4.1.1',
    'drmaa>=0.5',
    'ruffus>=2.2',
    'pybedtools>=0.6.2',
    'rdflib>=0.4.1',
    'hgapi>=1.3.0',
    'threadpool>=1.2.7',
    'PyYAML>=3.1.0',
    'pandas>=0.12.0',
    'weblogo>=3.0',
    'sphinxcontrib-programoutput>=0.8',
    'alignlib>=0.1']

cgat_packages= find_packages( exclude=["CGATPipelines*", "scripts*"])
# rename scripts
cgat_packages.append( "CGATScripts" )
cgat_package_dirs = { 'CGAT': 'CGAT',
                      'CGATScripts' : 'scripts' }

print cgat_packages

classifiers="""
Development Status :: 3 - Alpha
Intended Audience :: Science/Research
Intended Audience :: Developers
License :: OSI Approved
Programming Language :: Python
Topic :: Software Development
Topic :: Scientific/Engineering
Operating System :: Microsoft :: Windows
Operating System :: POSIX
Operating System :: Unix
Operating System :: MacOS
"""

# external dependencies
# R
# sqlite
# R - ggplot2
# R - RSqlite
# R - gplots (for r-heatmap)
# graphvis - for dependency graphs in documentation

##########################################################
## Extensions
# Connected components cython extension
Components = Extension(
     'CGAT.Components',                   
     [ 'CGAT/Components/Components.pyx',        
       'CGAT/Components/connected_components.cpp',],
     library_dirs=[],
     libraries=[],              
     language="c++",               
     )

# Nested containment lists
NCL = Extension(
    "CGAT.NCL.cnestedlist",                   
    [ "CGAT/NCL/cnestedlist.pyx",    
      "CGAT/NCL/intervaldb.c" ],
      library_dirs=[],
      libraries=[],
      language="c",
    )

# Nubiscan motif mapping
Nubiscan = Extension(
    "CGAT.Nubiscan.cnubiscan",                   
    [ 'CGAT/Nubiscan/cnubiscan.pyx'],
    library_dirs=[],
    libraries=[],            
    include_dirs = [numpy.get_include()], 
    language="c",               
 )

# Automatically build script extensions
pyx_files = glob.glob( "scripts/*.pyx" )
script_extensions = []
pysam_dirname = os.path.dirname( pysam.__file__ )
include_dirs = [ numpy.get_include() ] + pysam.get_include()
for pyx_file in pyx_files:
    script_name = os.path.basename( pyx_file )
    script_prefix = script_name[:-4]
    script_extensions.append( 
        Extension( "CGAT.%s" % (script_prefix),
                   sources = [pyx_file],
                   extra_link_args=[ os.path.join( pysam_dirname, "csamtools.so")],
                   include_dirs = include_dirs,
                   define_macros = pysam.get_defines() )
        )

setup(## package information
    name='CGAT',
    version='0.1.3',
    description='CGAT : the Computational Genomics Analysis Toolkit',
    author='Andreas Heger',
    author_email='andreas.heger@gmail.com',
    license="BSD",
    platforms=["any",],
    keywords="computational genomics",
    long_description='CGAT : the Computational Genomics Analysis Toolkit',
    classifiers = filter(None, classifiers.split("\n")),
    url="http://www.cgat.org/cgat/Tools/",
    ## package contents
    packages = cgat_packages, 
    package_dir= cgat_package_dirs,
    # package_data = { 'CGATScripts' : ['./scripts/*.py', './scripts/*.pyx', 
    #                                   './scripts/*.pyxbld', './scripts/*.pl' ],
    #                },

    include_package_data = True,

    entry_points = {
        'console_scripts': ['cgat = CGATScripts.cgat:main' ]
        },

    ## dependencies
    install_requires=shared_dependencies + extra_dependencies, 
    ## extension modules
    ext_modules=[Components, NCL, Nubiscan] + script_extensions,
    cmdclass = {'build_ext': build_ext},
    ## other options
    zip_safe = False,
    test_suite = "tests",
    )




################################################################################
#
#   MRC FGU Computational Genomics Group
#
#   $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $
#
#   Copyright (C) 2009 Andreas Heger
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of the GNU General Public License
#   as published by the Free Software Foundation; either version 2
#   of the License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#################################################################################
'''
beds2counts.py - compute overlap stats between multiple bed files
==================================================================

:Author: Nick Ilott 
:Release: $Id$
:Date: |today|
:Tags: Python

Purpose
-------

This script takes multiple bed files e.g. from multiple samples from the same experiment. It 
asseses the overlap between samples and outputs a count for each merged interval corresponding
to the number of samples that a particular interval was found in.

Writes the output to stdout.

Usage
-----

Example::

   python beds2counts.py file1.bed file2.bed > output.bed

Type::

   python beds2counts.py --help

for command line help.

Documentation
-------------

Code
----

'''
import tempfile
import os
import sys
import re
import optparse
import pybedtools
import CGAT.Experiment as E
import CGAT.Bed as Bed
import collections
import CGAT.IOTools as IOTools
import CGAT.IndexedGenome as IndexedGenome

def main( argv = None ):
    """script main.

    parses command line options in sys.argv, unless *argv* is given.
    """

    if not argv: argv = sys.argv

    # setup command line parser
    parser = E.OptionParser( version = "%prog version: $Id: cgat_script_template.py 2871 2010-03-03 10:20:44Z andreas $", 
                                    usage = globals()["__doc__"] )

    parser.add_option("-i", "--infiles", dest="infiles", type="string",
                      metavar = "bed",
                      action="append", help="supply list of bed files"  )

    parser.set_defaults( infiles = [] )
    ## add common options (-h/--help, ...) and parse command line 
    (options, args) = E.Start( parser, argv = argv )

    options.infiles.extend( args )
    if len(options.infiles) == 0:
        raise ValueError( 'please provide at least 1 bed file' )

    E.info("concatenating bed files")
    # concatenate the list of files
    tmp = tempfile.NamedTemporaryFile(delete = False)
    tmp_merge = tempfile.NamedTemporaryFile(delete = False)
    infs = options.infiles
    for inf in infs:
        for bed in Bed.iterator(IOTools.openFile(inf)):
            tmp.write("%s\n" % bed)
    tmp.close()
    
    E.info("merging bed entries")
    # merge the bed entries in the file
    name = tmp.name
    tmp_bed = pybedtools.BedTool(name)
    tmp_bed.merge().saveas(tmp_merge.name)
    tmp_merge.close()

    E.info("indexing bed entries")
    # index the bed entries                                                                                                                                                                                                                  
    merged = IndexedGenome.Simple()
    for bed in Bed.iterator(IOTools.openFile(tmp_merge.name)):
        merged.add(bed.contig, bed.start, bed.end)

    counts = collections.defaultdict(int)
    # list of samples                                                                                                                                                                                                                        
    samples = options.infiles
    
    E.info("counting no. samples overlapping each interval")
    for sample in samples:
        found = set()
        for bed in Bed.iterator(IOTools.openFile(sample)):
            if merged.contains(bed.contig, bed.start, bed.end):
                key =  [bed.contig] + [x for x in merged.get(bed.contig, bed.start, bed.end)]
                key = (key[0], key[1][0], key[1][1])
                if key in found: continue
                found.add(key)

                # tuple of interval description as key - (contig, start, end)                                                                                                                                                                
                counts[key] += 1

    # open outfile                                                                                                                                                                                                                           
    options.stdout.write("contig\tstart\tend\tcount\n")
    
    E.info("outputting result")
    for interval, count in counts.iteritems():
        options.stdout.write("\t".join(map(str,interval)) + "\t" + str(count) + "\n")

    ## write footer and output benchmark information.
    E.Stop()

if __name__ == "__main__":
    sys.exit( main( sys.argv) )

